//! For arithmetic operations which cannot trigger Illegal Behavior, this test evaluates those
//! operations with undefined operands (or partially-undefined vector operands), and then uses
//! `@compileLog` to validate the results are as expected in terms of comptime-known undefined.

export fn entry() void {
    testInt(u8);
    testInt(i8);
    testInt(u32);
    testInt(i32);
    testInt(u500);
    testInt(i500);

    testFloat(f16);
    testFloat(f32);
    testFloat(f64);
    testFloat(f80);
    testFloat(f128);
}

fn testInt(comptime Int: type) void {
    testIntWithValue(Int, 3); // all comptime-known
    testIntWithValue(Int, struct {
        var rt: Int = 0;
    }.rt); // partially runtime-known
}

fn testFloat(comptime Float: type) void {
    testFloatWithValue(Float, 3.0); // all comptime-known
    testFloatWithValue(Float, struct {
        var rt: Float = 0.0;
    }.rt); // partially runtime-known
}

inline fn testIntWithValue(comptime Int: type, x: Int) void {
    const V = @Vector(2, Int);
    const u: Int = undefined;

    // Wrapping addition

    @compileLog(x +% u); // undef
    @compileLog(u +% x); // undef

    @compileLog(V{ x, u } +% V{ x, x }); // { 6, undef }
    @compileLog(V{ u, x } +% V{ x, x }); // { undef, 6 }
    @compileLog(V{ u, u } +% V{ x, x }); // { undef, undef }

    @compileLog(V{ x, x } +% V{ x, u }); // { 6, undef }
    @compileLog(V{ x, u } +% V{ x, u }); // { 6, undef }
    @compileLog(V{ u, x } +% V{ x, u }); // { undef, undef }
    @compileLog(V{ u, u } +% V{ x, u }); // { undef, undef }

    @compileLog(V{ x, x } +% V{ u, x }); // { undef, 6 }
    @compileLog(V{ x, u } +% V{ u, x }); // { undef, undef }
    @compileLog(V{ u, x } +% V{ u, x }); // { undef, 6 }
    @compileLog(V{ u, u } +% V{ u, x }); // { undef, undef }

    @compileLog(V{ x, x } +% V{ u, u }); // { undef, undef }
    @compileLog(V{ x, u } +% V{ u, u }); // { undef, undef }
    @compileLog(V{ u, x } +% V{ u, u }); // { undef, undef }
    @compileLog(V{ u, u } +% V{ u, u }); // { undef, undef }

    // Saturating addition

    @compileLog(x +| u); // undef
    @compileLog(u +| x); // undef

    @compileLog(V{ x, u } +| V{ x, x }); // { 6, undef }
    @compileLog(V{ u, x } +| V{ x, x }); // { undef, 6 }
    @compileLog(V{ u, u } +| V{ x, x }); // { undef, undef }

    @compileLog(V{ x, x } +| V{ x, u }); // { 6, undef }
    @compileLog(V{ x, u } +| V{ x, u }); // { 6, undef }
    @compileLog(V{ u, x } +| V{ x, u }); // { undef, undef }
    @compileLog(V{ u, u } +| V{ x, u }); // { undef, undef }

    @compileLog(V{ x, x } +| V{ u, x }); // { undef, 6 }
    @compileLog(V{ x, u } +| V{ u, x }); // { undef, undef }
    @compileLog(V{ u, x } +| V{ u, x }); // { undef, 6 }
    @compileLog(V{ u, u } +| V{ u, x }); // { undef, undef }

    @compileLog(V{ x, x } +| V{ u, u }); // { undef, undef }
    @compileLog(V{ x, u } +| V{ u, u }); // { undef, undef }
    @compileLog(V{ u, x } +| V{ u, u }); // { undef, undef }
    @compileLog(V{ u, u } +| V{ u, u }); // { undef, undef }

    // Wrapping subtraction

    @compileLog(x -% u); // undef
    @compileLog(u -% x); // undef

    @compileLog(V{ x, u } -% V{ x, x }); // { 0, undef }
    @compileLog(V{ u, x } -% V{ x, x }); // { undef, 0 }
    @compileLog(V{ u, u } -% V{ x, x }); // { undef, undef }

    @compileLog(V{ x, x } -% V{ x, u }); // { 0, undef }
    @compileLog(V{ x, u } -% V{ x, u }); // { 0, undef }
    @compileLog(V{ u, x } -% V{ x, u }); // { undef, undef }
    @compileLog(V{ u, u } -% V{ x, u }); // { undef, undef }

    @compileLog(V{ x, x } -% V{ u, x }); // { undef, 0 }
    @compileLog(V{ x, u } -% V{ u, x }); // { undef, undef }
    @compileLog(V{ u, x } -% V{ u, x }); // { undef, 0 }
    @compileLog(V{ u, u } -% V{ u, x }); // { undef, undef }

    @compileLog(V{ x, x } -% V{ u, u }); // { undef, undef }
    @compileLog(V{ x, u } -% V{ u, u }); // { undef, undef }
    @compileLog(V{ u, x } -% V{ u, u }); // { undef, undef }
    @compileLog(V{ u, u } -% V{ u, u }); // { undef, undef }

    // Saturating subtraction

    @compileLog(x -| u); // undef
    @compileLog(u -| x); // undef

    @compileLog(V{ x, u } -| V{ x, x }); // { 0, undef }
    @compileLog(V{ u, x } -| V{ x, x }); // { undef, 0 }
    @compileLog(V{ u, u } -| V{ x, x }); // { undef, undef }

    @compileLog(V{ x, x } -| V{ x, u }); // { 0, undef }
    @compileLog(V{ x, u } -| V{ x, u }); // { 0, undef }
    @compileLog(V{ u, x } -| V{ x, u }); // { undef, undef }
    @compileLog(V{ u, u } -| V{ x, u }); // { undef, undef }

    @compileLog(V{ x, x } -| V{ u, x }); // { undef, 0 }
    @compileLog(V{ x, u } -| V{ u, x }); // { undef, undef }
    @compileLog(V{ u, x } -| V{ u, x }); // { undef, 0 }
    @compileLog(V{ u, u } -| V{ u, x }); // { undef, undef }

    @compileLog(V{ x, x } -| V{ u, u }); // { undef, undef }
    @compileLog(V{ x, u } -| V{ u, u }); // { undef, undef }
    @compileLog(V{ u, x } -| V{ u, u }); // { undef, undef }
    @compileLog(V{ u, u } -| V{ u, u }); // { undef, undef }

    // Wrapping multiplication

    @compileLog(x *% u); // undef
    @compileLog(u *% x); // undef

    @compileLog(V{ x, u } *% V{ x, x }); // { 9, undef }
    @compileLog(V{ u, x } *% V{ x, x }); // { undef, 9 }
    @compileLog(V{ u, u } *% V{ x, x }); // { undef, undef }

    @compileLog(V{ x, x } *% V{ x, u }); // { 9, undef }
    @compileLog(V{ x, u } *% V{ x, u }); // { 9, undef }
    @compileLog(V{ u, x } *% V{ x, u }); // { undef, undef }
    @compileLog(V{ u, u } *% V{ x, u }); // { undef, undef }

    @compileLog(V{ x, x } *% V{ u, x }); // { undef, 9 }
    @compileLog(V{ x, u } *% V{ u, x }); // { undef, undef }
    @compileLog(V{ u, x } *% V{ u, x }); // { undef, 9 }
    @compileLog(V{ u, u } *% V{ u, x }); // { undef, undef }

    @compileLog(V{ x, x } *% V{ u, u }); // { undef, undef }
    @compileLog(V{ x, u } *% V{ u, u }); // { undef, undef }
    @compileLog(V{ u, x } *% V{ u, u }); // { undef, undef }
    @compileLog(V{ u, u } *% V{ u, u }); // { undef, undef }

    // Saturating multiplication

    @compileLog(x *| u); // undef
    @compileLog(u *| x); // undef

    @compileLog(V{ x, u } *| V{ x, x }); // { 9, undef }
    @compileLog(V{ u, x } *| V{ x, x }); // { undef, 9 }
    @compileLog(V{ u, u } *| V{ x, x }); // { undef, undef }

    @compileLog(V{ x, x } *| V{ x, u }); // { 9, undef }
    @compileLog(V{ x, u } *| V{ x, u }); // { 9, undef }
    @compileLog(V{ u, x } *| V{ x, u }); // { undef, undef }
    @compileLog(V{ u, u } *| V{ x, u }); // { undef, undef }

    @compileLog(V{ x, x } *| V{ u, x }); // { undef, 9 }
    @compileLog(V{ x, u } *| V{ u, x }); // { undef, undef }
    @compileLog(V{ u, x } *| V{ u, x }); // { undef, 9 }
    @compileLog(V{ u, u } *| V{ u, x }); // { undef, undef }

    @compileLog(V{ x, x } *| V{ u, u }); // { undef, undef }
    @compileLog(V{ x, u } *| V{ u, u }); // { undef, undef }
    @compileLog(V{ u, x } *| V{ u, u }); // { undef, undef }
    @compileLog(V{ u, u } *| V{ u, u }); // { undef, undef }
}

inline fn testFloatWithValue(comptime Float: type, x: Float) void {
    const V = @Vector(2, Float);
    const u: Float = undefined;

    // Addition

    @compileLog(x + u); // undef
    @compileLog(u + x); // undef

    @compileLog(V{ x, u } + V{ x, x }); // { 6, undef }
    @compileLog(V{ u, x } + V{ x, x }); // { undef, 6 }
    @compileLog(V{ u, u } + V{ x, x }); // { undef, undef }

    @compileLog(V{ x, x } + V{ x, u }); // { 6, undef }
    @compileLog(V{ x, u } + V{ x, u }); // { 6, undef }
    @compileLog(V{ u, x } + V{ x, u }); // { undef, undef }
    @compileLog(V{ u, u } + V{ x, u }); // { undef, undef }

    @compileLog(V{ x, x } + V{ u, x }); // { undef, 6 }
    @compileLog(V{ x, u } + V{ u, x }); // { undef, undef }
    @compileLog(V{ u, x } + V{ u, x }); // { undef, 6 }
    @compileLog(V{ u, u } + V{ u, x }); // { undef, undef }

    @compileLog(V{ x, x } + V{ u, u }); // { undef, undef }
    @compileLog(V{ x, u } + V{ u, u }); // { undef, undef }
    @compileLog(V{ u, x } + V{ u, u }); // { undef, undef }
    @compileLog(V{ u, u } + V{ u, u }); // { undef, undef }

    // Subtraction

    @compileLog(x - u); // undef
    @compileLog(u - x); // undef

    @compileLog(V{ x, u } - V{ x, x }); // { 0, undef }
    @compileLog(V{ u, x } - V{ x, x }); // { undef, 0 }
    @compileLog(V{ u, u } - V{ x, x }); // { undef, undef }

    @compileLog(V{ x, x } - V{ x, u }); // { 0, undef }
    @compileLog(V{ x, u } - V{ x, u }); // { 0, undef }
    @compileLog(V{ u, x } - V{ x, u }); // { undef, undef }
    @compileLog(V{ u, u } - V{ x, u }); // { undef, undef }

    @compileLog(V{ x, x } - V{ u, x }); // { undef, 0 }
    @compileLog(V{ x, u } - V{ u, x }); // { undef, undef }
    @compileLog(V{ u, x } - V{ u, x }); // { undef, 0 }
    @compileLog(V{ u, u } - V{ u, x }); // { undef, undef }

    @compileLog(V{ x, x } - V{ u, u }); // { undef, undef }
    @compileLog(V{ x, u } - V{ u, u }); // { undef, undef }
    @compileLog(V{ u, x } - V{ u, u }); // { undef, undef }
    @compileLog(V{ u, u } - V{ u, u }); // { undef, undef }

    // Multiplication

    @compileLog(x * u); // undef
    @compileLog(u * x); // undef

    @compileLog(V{ x, u } * V{ x, x }); // { 9, undef }
    @compileLog(V{ u, x } * V{ x, x }); // { undef, 9 }
    @compileLog(V{ u, u } * V{ x, x }); // { undef, undef }

    @compileLog(V{ x, x } * V{ x, u }); // { 9, undef }
    @compileLog(V{ x, u } * V{ x, u }); // { 9, undef }
    @compileLog(V{ u, x } * V{ x, u }); // { undef, undef }
    @compileLog(V{ u, u } * V{ x, u }); // { undef, undef }

    @compileLog(V{ x, x } * V{ u, x }); // { undef, 9 }
    @compileLog(V{ x, u } * V{ u, x }); // { undef, undef }
    @compileLog(V{ u, x } * V{ u, x }); // { undef, 9 }
    @compileLog(V{ u, u } * V{ u, x }); // { undef, undef }

    @compileLog(V{ x, x } * V{ u, u }); // { undef, undef }
    @compileLog(V{ x, u } * V{ u, u }); // { undef, undef }
    @compileLog(V{ u, x } * V{ u, u }); // { undef, undef }
    @compileLog(V{ u, u } * V{ u, u }); // { undef, undef }

    // Negation

    @compileLog(-u); // undef
    @compileLog(-V{ x, u }); // { -3, undef }
    @compileLog(-V{ u, x }); // { undef, -3 }
    @compileLog(-V{ u, u }); // { undef, undef }
}

// error
//
// :40:5: error: found compile log statement
// :40:5: note: also here (5 times)
// :189:5: note: also here (5 times)
//
// Compile Log Output:
// @as(u8, undefined)
// @as(u8, undefined)
// @as(@Vector(2, u8), .{ 6, undefined })
// @as(@Vector(2, u8), .{ undefined, 6 })
// @as(@Vector(2, u8), .{ undefined, undefined })
// @as(@Vector(2, u8), .{ 6, undefined })
// @as(@Vector(2, u8), .{ 6, undefined })
// @as(@Vector(2, u8), .{ undefined, undefined })
// @as(@Vector(2, u8), .{ undefined, undefined })
// @as(@Vector(2, u8), .{ undefined, 6 })
// @as(@Vector(2, u8), .{ undefined, undefined })
// @as(@Vector(2, u8), .{ undefined, 6 })
// @as(@Vector(2, u8), .{ undefined, undefined })
// @as(@Vector(2, u8), .{ undefined, undefined })
// @as(@Vector(2, u8), .{ undefined, undefined })
// @as(@Vector(2, u8), .{ undefined, undefined })
// @as(@Vector(2, u8), .{ undefined, undefined })
// @as(u8, undefined)
// @as(u8, undefined)
// @as(@Vector(2, u8), .{ 6, undefined })
// @as(@Vector(2, u8), .{ undefined, 6 })
// @as(@Vector(2, u8), .{ undefined, undefined })
// @as(@Vector(2, u8), .{ 6, undefined })
// @as(@Vector(2, u8), .{ 6, undefined })
// @as(@Vector(2, u8), .{ undefined, undefined })
// @as(@Vector(2, u8), .{ undefined, undefined })
// @as(@Vector(2, u8), .{ undefined, 6 })
// @as(@Vector(2, u8), .{ undefined, undefined })
// @as(@Vector(2, u8), .{ undefined, 6 })
// @as(@Vector(2, u8), .{ undefined, undefined })
// @as(@Vector(2, u8), .{ undefined, undefined })
// @as(@Vector(2, u8), .{ undefined, undefined })
// @as(@Vector(2, u8), .{ undefined, undefined })
// @as(@Vector(2, u8), .{ undefined, undefined })
// @as(u8, undefined)
// @as(u8, undefined)
// @as(@Vector(2, u8), .{ 0, undefined })
// @as(@Vector(2, u8), .{ undefined, 0 })
// @as(@Vector(2, u8), .{ undefined, undefined })
// @as(@Vector(2, u8), .{ 0, undefined })
// @as(@Vector(2, u8), .{ 0, undefined })
// @as(@Vector(2, u8), .{ undefined, undefined })
// @as(@Vector(2, u8), .{ undefined, undefined })
// @as(@Vector(2, u8), .{ undefined, 0 })
// @as(@Vector(2, u8), .{ undefined, undefined })
// @as(@Vector(2, u8), .{ undefined, 0 })
// @as(@Vector(2, u8), .{ undefined, undefined })
// @as(@Vector(2, u8), .{ undefined, undefined })
// @as(@Vector(2, u8), .{ undefined, undefined })
// @as(@Vector(2, u8), .{ undefined, undefined })
// @as(@Vector(2, u8), .{ undefined, undefined })
// @as(u8, undefined)
// @as(u8, undefined)
// @as(@Vector(2, u8), .{ 0, undefined })
// @as(@Vector(2, u8), .{ undefined, 0 })
// @as(@Vector(2, u8), .{ undefined, undefined })
// @as(@Vector(2, u8), .{ 0, undefined })
// @as(@Vector(2, u8), .{ 0, undefined })
// @as(@Vector(2, u8), .{ undefined, undefined })
// @as(@Vector(2, u8), .{ undefined, undefined })
// @as(@Vector(2, u8), .{ undefined, 0 })
// @as(@Vector(2, u8), .{ undefined, undefined })
// @as(@Vector(2, u8), .{ undefined, 0 })
// @as(@Vector(2, u8), .{ undefined, undefined })
// @as(@Vector(2, u8), .{ undefined, undefined })
// @as(@Vector(2, u8), .{ undefined, undefined })
// @as(@Vector(2, u8), .{ undefined, undefined })
// @as(@Vector(2, u8), .{ undefined, undefined })
// @as(u8, undefined)
// @as(u8, undefined)
// @as(@Vector(2, u8), .{ 9, undefined })
// @as(@Vector(2, u8), .{ undefined, 9 })
// @as(@Vector(2, u8), .{ undefined, undefined })
// @as(@Vector(2, u8), .{ 9, undefined })
// @as(@Vector(2, u8), .{ 9, undefined })
// @as(@Vector(2, u8), .{ undefined, undefined })
// @as(@Vector(2, u8), .{ undefined, undefined })
// @as(@Vector(2, u8), .{ undefined, 9 })
// @as(@Vector(2, u8), .{ undefined, undefined })
// @as(@Vector(2, u8), .{ undefined, 9 })
// @as(@Vector(2, u8), .{ undefined, undefined })
// @as(@Vector(2, u8), .{ undefined, undefined })
// @as(@Vector(2, u8), .{ undefined, undefined })
// @as(@Vector(2, u8), .{ undefined, undefined })
// @as(@Vector(2, u8), .{ undefined, undefined })
// @as(u8, undefined)
// @as(u8, undefined)
// @as(@Vector(2, u8), .{ 9, undefined })
// @as(@Vector(2, u8), .{ undefined, 9 })
// @as(@Vector(2, u8), .{ undefined, undefined })
// @as(@Vector(2, u8), .{ 9, undefined })
// @as(@Vector(2, u8), .{ 9, undefined })
// @as(@Vector(2, u8), .{ undefined, undefined })
// @as(@Vector(2, u8), .{ undefined, undefined })
// @as(@Vector(2, u8), .{ undefined, 9 })
// @as(@Vector(2, u8), .{ undefined, undefined })
// @as(@Vector(2, u8), .{ undefined, 9 })
// @as(@Vector(2, u8), .{ undefined, undefined })
// @as(@Vector(2, u8), .{ undefined, undefined })
// @as(@Vector(2, u8), .{ undefined, undefined })
// @as(@Vector(2, u8), .{ undefined, undefined })
// @as(@Vector(2, u8), .{ undefined, undefined })
// @as(u8, undefined)
// @as(u8, undefined)
// @as(@Vector(2, u8), [runtime value])
// @as(@Vector(2, u8), [runtime value])
// @as(@Vector(2, u8), [runtime value])
// @as(@Vector(2, u8), [runtime value])
// @as(@Vector(2, u8), [runtime value])
// @as(@Vector(2, u8), [runtime value])
// @as(@Vector(2, u8), [runtime value])
// @as(@Vector(2, u8), [runtime value])
// @as(@Vector(2, u8), [runtime value])
// @as(@Vector(2, u8), [runtime value])
// @as(@Vector(2, u8), [runtime value])
// @as(@Vector(2, u8), [runtime value])
// @as(@Vector(2, u8), [runtime value])
// @as(@Vector(2, u8), [runtime value])
// @as(@Vector(2, u8), .{ undefined, undefined })
// @as(u8, undefined)
// @as(u8, undefined)
// @as(@Vector(2, u8), [runtime value])
// @as(@Vector(2, u8), [runtime value])
// @as(@Vector(2, u8), [runtime value])
// @as(@Vector(2, u8), [runtime value])
// @as(@Vector(2, u8), [runtime value])
// @as(@Vector(2, u8), [runtime value])
// @as(@Vector(2, u8), [runtime value])
// @as(@Vector(2, u8), [runtime value])
// @as(@Vector(2, u8), [runtime value])
// @as(@Vector(2, u8), [runtime value])
// @as(@Vector(2, u8), [runtime value])
// @as(@Vector(2, u8), [runtime value])
// @as(@Vector(2, u8), [runtime value])
// @as(@Vector(2, u8), [runtime value])
// @as(@Vector(2, u8), .{ undefined, undefined })
// @as(u8, undefined)
// @as(u8, undefined)
// @as(@Vector(2, u8), [runtime value])
// @as(@Vector(2, u8), [runtime value])
// @as(@Vector(2, u8), [runtime value])
// @as(@Vector(2, u8), [runtime value])
// @as(@Vector(2, u8), [runtime value])
// @as(@Vector(2, u8), [runtime value])
// @as(@Vector(2, u8), [runtime value])
// @as(@Vector(2, u8), [runtime value])
// @as(@Vector(2, u8), [runtime value])
// @as(@Vector(2, u8), [runtime value])
// @as(@Vector(2, u8), [runtime value])
// @as(@Vector(2, u8), [runtime value])
// @as(@Vector(2, u8), [runtime value])
// @as(@Vector(2, u8), [runtime value])
// @as(@Vector(2, u8), .{ undefined, undefined })
// @as(u8, undefined)
// @as(u8, undefined)
// @as(@Vector(2, u8), [runtime value])
// @as(@Vector(2, u8), [runtime value])
// @as(@Vector(2, u8), [runtime value])
// @as(@Vector(2, u8), [runtime value])
// @as(@Vector(2, u8), [runtime value])
// @as(@Vector(2, u8), [runtime value])
// @as(@Vector(2, u8), [runtime value])
// @as(@Vector(2, u8), [runtime value])
// @as(@Vector(2, u8), [runtime value])
// @as(@Vector(2, u8), [runtime value])
// @as(@Vector(2, u8), [runtime value])
// @as(@Vector(2, u8), [runtime value])
// @as(@Vector(2, u8), [runtime value])
// @as(@Vector(2, u8), [runtime value])
// @as(@Vector(2, u8), .{ undefined, undefined })
// @as(u8, undefined)
// @as(u8, undefined)
// @as(@Vector(2, u8), [runtime value])
// @as(@Vector(2, u8), [runtime value])
// @as(@Vector(2, u8), [runtime value])
// @as(@Vector(2, u8), [runtime value])
// @as(@Vector(2, u8), [runtime value])
// @as(@Vector(2, u8), [runtime value])
// @as(@Vector(2, u8), [runtime value])
// @as(@Vector(2, u8), [runtime value])
// @as(@Vector(2, u8), [runtime value])
// @as(@Vector(2, u8), [runtime value])
// @as(@Vector(2, u8), [runtime value])
// @as(@Vector(2, u8), [runtime value])
// @as(@Vector(2, u8), [runtime value])
// @as(@Vector(2, u8), [runtime value])
// @as(@Vector(2, u8), .{ undefined, undefined })
// @as(u8, undefined)
// @as(u8, undefined)
// @as(@Vector(2, u8), [runtime value])
// @as(@Vector(2, u8), [runtime value])
// @as(@Vector(2, u8), [runtime value])
// @as(@Vector(2, u8), [runtime value])
// @as(@Vector(2, u8), [runtime value])
// @as(@Vector(2, u8), [runtime value])
// @as(@Vector(2, u8), [runtime value])
// @as(@Vector(2, u8), [runtime value])
// @as(@Vector(2, u8), [runtime value])
// @as(@Vector(2, u8), [runtime value])
// @as(@Vector(2, u8), [runtime value])
// @as(@Vector(2, u8), [runtime value])
// @as(@Vector(2, u8), [runtime value])
// @as(@Vector(2, u8), [runtime value])
// @as(@Vector(2, u8), .{ undefined, undefined })
// @as(i8, undefined)
// @as(i8, undefined)
// @as(@Vector(2, i8), .{ 6, undefined })
// @as(@Vector(2, i8), .{ undefined, 6 })
// @as(@Vector(2, i8), .{ undefined, undefined })
// @as(@Vector(2, i8), .{ 6, undefined })
// @as(@Vector(2, i8), .{ 6, undefined })
// @as(@Vector(2, i8), .{ undefined, undefined })
// @as(@Vector(2, i8), .{ undefined, undefined })
// @as(@Vector(2, i8), .{ undefined, 6 })
// @as(@Vector(2, i8), .{ undefined, undefined })
// @as(@Vector(2, i8), .{ undefined, 6 })
// @as(@Vector(2, i8), .{ undefined, undefined })
// @as(@Vector(2, i8), .{ undefined, undefined })
// @as(@Vector(2, i8), .{ undefined, undefined })
// @as(@Vector(2, i8), .{ undefined, undefined })
// @as(@Vector(2, i8), .{ undefined, undefined })
// @as(i8, undefined)
// @as(i8, undefined)
// @as(@Vector(2, i8), .{ 6, undefined })
// @as(@Vector(2, i8), .{ undefined, 6 })
// @as(@Vector(2, i8), .{ undefined, undefined })
// @as(@Vector(2, i8), .{ 6, undefined })
// @as(@Vector(2, i8), .{ 6, undefined })
// @as(@Vector(2, i8), .{ undefined, undefined })
// @as(@Vector(2, i8), .{ undefined, undefined })
// @as(@Vector(2, i8), .{ undefined, 6 })
// @as(@Vector(2, i8), .{ undefined, undefined })
// @as(@Vector(2, i8), .{ undefined, 6 })
// @as(@Vector(2, i8), .{ undefined, undefined })
// @as(@Vector(2, i8), .{ undefined, undefined })
// @as(@Vector(2, i8), .{ undefined, undefined })
// @as(@Vector(2, i8), .{ undefined, undefined })
// @as(@Vector(2, i8), .{ undefined, undefined })
// @as(i8, undefined)
// @as(i8, undefined)
// @as(@Vector(2, i8), .{ 0, undefined })
// @as(@Vector(2, i8), .{ undefined, 0 })
// @as(@Vector(2, i8), .{ undefined, undefined })
// @as(@Vector(2, i8), .{ 0, undefined })
// @as(@Vector(2, i8), .{ 0, undefined })
// @as(@Vector(2, i8), .{ undefined, undefined })
// @as(@Vector(2, i8), .{ undefined, undefined })
// @as(@Vector(2, i8), .{ undefined, 0 })
// @as(@Vector(2, i8), .{ undefined, undefined })
// @as(@Vector(2, i8), .{ undefined, 0 })
// @as(@Vector(2, i8), .{ undefined, undefined })
// @as(@Vector(2, i8), .{ undefined, undefined })
// @as(@Vector(2, i8), .{ undefined, undefined })
// @as(@Vector(2, i8), .{ undefined, undefined })
// @as(@Vector(2, i8), .{ undefined, undefined })
// @as(i8, undefined)
// @as(i8, undefined)
// @as(@Vector(2, i8), .{ 0, undefined })
// @as(@Vector(2, i8), .{ undefined, 0 })
// @as(@Vector(2, i8), .{ undefined, undefined })
// @as(@Vector(2, i8), .{ 0, undefined })
// @as(@Vector(2, i8), .{ 0, undefined })
// @as(@Vector(2, i8), .{ undefined, undefined })
// @as(@Vector(2, i8), .{ undefined, undefined })
// @as(@Vector(2, i8), .{ undefined, 0 })
// @as(@Vector(2, i8), .{ undefined, undefined })
// @as(@Vector(2, i8), .{ undefined, 0 })
// @as(@Vector(2, i8), .{ undefined, undefined })
// @as(@Vector(2, i8), .{ undefined, undefined })
// @as(@Vector(2, i8), .{ undefined, undefined })
// @as(@Vector(2, i8), .{ undefined, undefined })
// @as(@Vector(2, i8), .{ undefined, undefined })
// @as(i8, undefined)
// @as(i8, undefined)
// @as(@Vector(2, i8), .{ 9, undefined })
// @as(@Vector(2, i8), .{ undefined, 9 })
// @as(@Vector(2, i8), .{ undefined, undefined })
// @as(@Vector(2, i8), .{ 9, undefined })
// @as(@Vector(2, i8), .{ 9, undefined })
// @as(@Vector(2, i8), .{ undefined, undefined })
// @as(@Vector(2, i8), .{ undefined, undefined })
// @as(@Vector(2, i8), .{ undefined, 9 })
// @as(@Vector(2, i8), .{ undefined, undefined })
// @as(@Vector(2, i8), .{ undefined, 9 })
// @as(@Vector(2, i8), .{ undefined, undefined })
// @as(@Vector(2, i8), .{ undefined, undefined })
// @as(@Vector(2, i8), .{ undefined, undefined })
// @as(@Vector(2, i8), .{ undefined, undefined })
// @as(@Vector(2, i8), .{ undefined, undefined })
// @as(i8, undefined)
// @as(i8, undefined)
// @as(@Vector(2, i8), .{ 9, undefined })
// @as(@Vector(2, i8), .{ undefined, 9 })
// @as(@Vector(2, i8), .{ undefined, undefined })
// @as(@Vector(2, i8), .{ 9, undefined })
// @as(@Vector(2, i8), .{ 9, undefined })
// @as(@Vector(2, i8), .{ undefined, undefined })
// @as(@Vector(2, i8), .{ undefined, undefined })
// @as(@Vector(2, i8), .{ undefined, 9 })
// @as(@Vector(2, i8), .{ undefined, undefined })
// @as(@Vector(2, i8), .{ undefined, 9 })
// @as(@Vector(2, i8), .{ undefined, undefined })
// @as(@Vector(2, i8), .{ undefined, undefined })
// @as(@Vector(2, i8), .{ undefined, undefined })
// @as(@Vector(2, i8), .{ undefined, undefined })
// @as(@Vector(2, i8), .{ undefined, undefined })
// @as(i8, undefined)
// @as(i8, undefined)
// @as(@Vector(2, i8), [runtime value])
// @as(@Vector(2, i8), [runtime value])
// @as(@Vector(2, i8), [runtime value])
// @as(@Vector(2, i8), [runtime value])
// @as(@Vector(2, i8), [runtime value])
// @as(@Vector(2, i8), [runtime value])
// @as(@Vector(2, i8), [runtime value])
// @as(@Vector(2, i8), [runtime value])
// @as(@Vector(2, i8), [runtime value])
// @as(@Vector(2, i8), [runtime value])
// @as(@Vector(2, i8), [runtime value])
// @as(@Vector(2, i8), [runtime value])
// @as(@Vector(2, i8), [runtime value])
// @as(@Vector(2, i8), [runtime value])
// @as(@Vector(2, i8), .{ undefined, undefined })
// @as(i8, undefined)
// @as(i8, undefined)
// @as(@Vector(2, i8), [runtime value])
// @as(@Vector(2, i8), [runtime value])
// @as(@Vector(2, i8), [runtime value])
// @as(@Vector(2, i8), [runtime value])
// @as(@Vector(2, i8), [runtime value])
// @as(@Vector(2, i8), [runtime value])
// @as(@Vector(2, i8), [runtime value])
// @as(@Vector(2, i8), [runtime value])
// @as(@Vector(2, i8), [runtime value])
// @as(@Vector(2, i8), [runtime value])
// @as(@Vector(2, i8), [runtime value])
// @as(@Vector(2, i8), [runtime value])
// @as(@Vector(2, i8), [runtime value])
// @as(@Vector(2, i8), [runtime value])
// @as(@Vector(2, i8), .{ undefined, undefined })
// @as(i8, undefined)
// @as(i8, undefined)
// @as(@Vector(2, i8), [runtime value])
// @as(@Vector(2, i8), [runtime value])
// @as(@Vector(2, i8), [runtime value])
// @as(@Vector(2, i8), [runtime value])
// @as(@Vector(2, i8), [runtime value])
// @as(@Vector(2, i8), [runtime value])
// @as(@Vector(2, i8), [runtime value])
// @as(@Vector(2, i8), [runtime value])
// @as(@Vector(2, i8), [runtime value])
// @as(@Vector(2, i8), [runtime value])
// @as(@Vector(2, i8), [runtime value])
// @as(@Vector(2, i8), [runtime value])
// @as(@Vector(2, i8), [runtime value])
// @as(@Vector(2, i8), [runtime value])
// @as(@Vector(2, i8), .{ undefined, undefined })
// @as(i8, undefined)
// @as(i8, undefined)
// @as(@Vector(2, i8), [runtime value])
// @as(@Vector(2, i8), [runtime value])
// @as(@Vector(2, i8), [runtime value])
// @as(@Vector(2, i8), [runtime value])
// @as(@Vector(2, i8), [runtime value])
// @as(@Vector(2, i8), [runtime value])
// @as(@Vector(2, i8), [runtime value])
// @as(@Vector(2, i8), [runtime value])
// @as(@Vector(2, i8), [runtime value])
// @as(@Vector(2, i8), [runtime value])
// @as(@Vector(2, i8), [runtime value])
// @as(@Vector(2, i8), [runtime value])
// @as(@Vector(2, i8), [runtime value])
// @as(@Vector(2, i8), [runtime value])
// @as(@Vector(2, i8), .{ undefined, undefined })
// @as(i8, undefined)
// @as(i8, undefined)
// @as(@Vector(2, i8), [runtime value])
// @as(@Vector(2, i8), [runtime value])
// @as(@Vector(2, i8), [runtime value])
// @as(@Vector(2, i8), [runtime value])
// @as(@Vector(2, i8), [runtime value])
// @as(@Vector(2, i8), [runtime value])
// @as(@Vector(2, i8), [runtime value])
// @as(@Vector(2, i8), [runtime value])
// @as(@Vector(2, i8), [runtime value])
// @as(@Vector(2, i8), [runtime value])
// @as(@Vector(2, i8), [runtime value])
// @as(@Vector(2, i8), [runtime value])
// @as(@Vector(2, i8), [runtime value])
// @as(@Vector(2, i8), [runtime value])
// @as(@Vector(2, i8), .{ undefined, undefined })
// @as(i8, undefined)
// @as(i8, undefined)
// @as(@Vector(2, i8), [runtime value])
// @as(@Vector(2, i8), [runtime value])
// @as(@Vector(2, i8), [runtime value])
// @as(@Vector(2, i8), [runtime value])
// @as(@Vector(2, i8), [runtime value])
// @as(@Vector(2, i8), [runtime value])
// @as(@Vector(2, i8), [runtime value])
// @as(@Vector(2, i8), [runtime value])
// @as(@Vector(2, i8), [runtime value])
// @as(@Vector(2, i8), [runtime value])
// @as(@Vector(2, i8), [runtime value])
// @as(@Vector(2, i8), [runtime value])
// @as(@Vector(2, i8), [runtime value])
// @as(@Vector(2, i8), [runtime value])
// @as(@Vector(2, i8), .{ undefined, undefined })
// @as(u32, undefined)
// @as(u32, undefined)
// @as(@Vector(2, u32), .{ 6, undefined })
// @as(@Vector(2, u32), .{ undefined, 6 })
// @as(@Vector(2, u32), .{ undefined, undefined })
// @as(@Vector(2, u32), .{ 6, undefined })
// @as(@Vector(2, u32), .{ 6, undefined })
// @as(@Vector(2, u32), .{ undefined, undefined })
// @as(@Vector(2, u32), .{ undefined, undefined })
// @as(@Vector(2, u32), .{ undefined, 6 })
// @as(@Vector(2, u32), .{ undefined, undefined })
// @as(@Vector(2, u32), .{ undefined, 6 })
// @as(@Vector(2, u32), .{ undefined, undefined })
// @as(@Vector(2, u32), .{ undefined, undefined })
// @as(@Vector(2, u32), .{ undefined, undefined })
// @as(@Vector(2, u32), .{ undefined, undefined })
// @as(@Vector(2, u32), .{ undefined, undefined })
// @as(u32, undefined)
// @as(u32, undefined)
// @as(@Vector(2, u32), .{ 6, undefined })
// @as(@Vector(2, u32), .{ undefined, 6 })
// @as(@Vector(2, u32), .{ undefined, undefined })
// @as(@Vector(2, u32), .{ 6, undefined })
// @as(@Vector(2, u32), .{ 6, undefined })
// @as(@Vector(2, u32), .{ undefined, undefined })
// @as(@Vector(2, u32), .{ undefined, undefined })
// @as(@Vector(2, u32), .{ undefined, 6 })
// @as(@Vector(2, u32), .{ undefined, undefined })
// @as(@Vector(2, u32), .{ undefined, 6 })
// @as(@Vector(2, u32), .{ undefined, undefined })
// @as(@Vector(2, u32), .{ undefined, undefined })
// @as(@Vector(2, u32), .{ undefined, undefined })
// @as(@Vector(2, u32), .{ undefined, undefined })
// @as(@Vector(2, u32), .{ undefined, undefined })
// @as(u32, undefined)
// @as(u32, undefined)
// @as(@Vector(2, u32), .{ 0, undefined })
// @as(@Vector(2, u32), .{ undefined, 0 })
// @as(@Vector(2, u32), .{ undefined, undefined })
// @as(@Vector(2, u32), .{ 0, undefined })
// @as(@Vector(2, u32), .{ 0, undefined })
// @as(@Vector(2, u32), .{ undefined, undefined })
// @as(@Vector(2, u32), .{ undefined, undefined })
// @as(@Vector(2, u32), .{ undefined, 0 })
// @as(@Vector(2, u32), .{ undefined, undefined })
// @as(@Vector(2, u32), .{ undefined, 0 })
// @as(@Vector(2, u32), .{ undefined, undefined })
// @as(@Vector(2, u32), .{ undefined, undefined })
// @as(@Vector(2, u32), .{ undefined, undefined })
// @as(@Vector(2, u32), .{ undefined, undefined })
// @as(@Vector(2, u32), .{ undefined, undefined })
// @as(u32, undefined)
// @as(u32, undefined)
// @as(@Vector(2, u32), .{ 0, undefined })
// @as(@Vector(2, u32), .{ undefined, 0 })
// @as(@Vector(2, u32), .{ undefined, undefined })
// @as(@Vector(2, u32), .{ 0, undefined })
// @as(@Vector(2, u32), .{ 0, undefined })
// @as(@Vector(2, u32), .{ undefined, undefined })
// @as(@Vector(2, u32), .{ undefined, undefined })
// @as(@Vector(2, u32), .{ undefined, 0 })
// @as(@Vector(2, u32), .{ undefined, undefined })
// @as(@Vector(2, u32), .{ undefined, 0 })
// @as(@Vector(2, u32), .{ undefined, undefined })
// @as(@Vector(2, u32), .{ undefined, undefined })
// @as(@Vector(2, u32), .{ undefined, undefined })
// @as(@Vector(2, u32), .{ undefined, undefined })
// @as(@Vector(2, u32), .{ undefined, undefined })
// @as(u32, undefined)
// @as(u32, undefined)
// @as(@Vector(2, u32), .{ 9, undefined })
// @as(@Vector(2, u32), .{ undefined, 9 })
// @as(@Vector(2, u32), .{ undefined, undefined })
// @as(@Vector(2, u32), .{ 9, undefined })
// @as(@Vector(2, u32), .{ 9, undefined })
// @as(@Vector(2, u32), .{ undefined, undefined })
// @as(@Vector(2, u32), .{ undefined, undefined })
// @as(@Vector(2, u32), .{ undefined, 9 })
// @as(@Vector(2, u32), .{ undefined, undefined })
// @as(@Vector(2, u32), .{ undefined, 9 })
// @as(@Vector(2, u32), .{ undefined, undefined })
// @as(@Vector(2, u32), .{ undefined, undefined })
// @as(@Vector(2, u32), .{ undefined, undefined })
// @as(@Vector(2, u32), .{ undefined, undefined })
// @as(@Vector(2, u32), .{ undefined, undefined })
// @as(u32, undefined)
// @as(u32, undefined)
// @as(@Vector(2, u32), .{ 9, undefined })
// @as(@Vector(2, u32), .{ undefined, 9 })
// @as(@Vector(2, u32), .{ undefined, undefined })
// @as(@Vector(2, u32), .{ 9, undefined })
// @as(@Vector(2, u32), .{ 9, undefined })
// @as(@Vector(2, u32), .{ undefined, undefined })
// @as(@Vector(2, u32), .{ undefined, undefined })
// @as(@Vector(2, u32), .{ undefined, 9 })
// @as(@Vector(2, u32), .{ undefined, undefined })
// @as(@Vector(2, u32), .{ undefined, 9 })
// @as(@Vector(2, u32), .{ undefined, undefined })
// @as(@Vector(2, u32), .{ undefined, undefined })
// @as(@Vector(2, u32), .{ undefined, undefined })
// @as(@Vector(2, u32), .{ undefined, undefined })
// @as(@Vector(2, u32), .{ undefined, undefined })
// @as(u32, undefined)
// @as(u32, undefined)
// @as(@Vector(2, u32), [runtime value])
// @as(@Vector(2, u32), [runtime value])
// @as(@Vector(2, u32), [runtime value])
// @as(@Vector(2, u32), [runtime value])
// @as(@Vector(2, u32), [runtime value])
// @as(@Vector(2, u32), [runtime value])
// @as(@Vector(2, u32), [runtime value])
// @as(@Vector(2, u32), [runtime value])
// @as(@Vector(2, u32), [runtime value])
// @as(@Vector(2, u32), [runtime value])
// @as(@Vector(2, u32), [runtime value])
// @as(@Vector(2, u32), [runtime value])
// @as(@Vector(2, u32), [runtime value])
// @as(@Vector(2, u32), [runtime value])
// @as(@Vector(2, u32), .{ undefined, undefined })
// @as(u32, undefined)
// @as(u32, undefined)
// @as(@Vector(2, u32), [runtime value])
// @as(@Vector(2, u32), [runtime value])
// @as(@Vector(2, u32), [runtime value])
// @as(@Vector(2, u32), [runtime value])
// @as(@Vector(2, u32), [runtime value])
// @as(@Vector(2, u32), [runtime value])
// @as(@Vector(2, u32), [runtime value])
// @as(@Vector(2, u32), [runtime value])
// @as(@Vector(2, u32), [runtime value])
// @as(@Vector(2, u32), [runtime value])
// @as(@Vector(2, u32), [runtime value])
// @as(@Vector(2, u32), [runtime value])
// @as(@Vector(2, u32), [runtime value])
// @as(@Vector(2, u32), [runtime value])
// @as(@Vector(2, u32), .{ undefined, undefined })
// @as(u32, undefined)
// @as(u32, undefined)
// @as(@Vector(2, u32), [runtime value])
// @as(@Vector(2, u32), [runtime value])
// @as(@Vector(2, u32), [runtime value])
// @as(@Vector(2, u32), [runtime value])
// @as(@Vector(2, u32), [runtime value])
// @as(@Vector(2, u32), [runtime value])
// @as(@Vector(2, u32), [runtime value])
// @as(@Vector(2, u32), [runtime value])
// @as(@Vector(2, u32), [runtime value])
// @as(@Vector(2, u32), [runtime value])
// @as(@Vector(2, u32), [runtime value])
// @as(@Vector(2, u32), [runtime value])
// @as(@Vector(2, u32), [runtime value])
// @as(@Vector(2, u32), [runtime value])
// @as(@Vector(2, u32), .{ undefined, undefined })
// @as(u32, undefined)
// @as(u32, undefined)
// @as(@Vector(2, u32), [runtime value])
// @as(@Vector(2, u32), [runtime value])
// @as(@Vector(2, u32), [runtime value])
// @as(@Vector(2, u32), [runtime value])
// @as(@Vector(2, u32), [runtime value])
// @as(@Vector(2, u32), [runtime value])
// @as(@Vector(2, u32), [runtime value])
// @as(@Vector(2, u32), [runtime value])
// @as(@Vector(2, u32), [runtime value])
// @as(@Vector(2, u32), [runtime value])
// @as(@Vector(2, u32), [runtime value])
// @as(@Vector(2, u32), [runtime value])
// @as(@Vector(2, u32), [runtime value])
// @as(@Vector(2, u32), [runtime value])
// @as(@Vector(2, u32), .{ undefined, undefined })
// @as(u32, undefined)
// @as(u32, undefined)
// @as(@Vector(2, u32), [runtime value])
// @as(@Vector(2, u32), [runtime value])
// @as(@Vector(2, u32), [runtime value])
// @as(@Vector(2, u32), [runtime value])
// @as(@Vector(2, u32), [runtime value])
// @as(@Vector(2, u32), [runtime value])
// @as(@Vector(2, u32), [runtime value])
// @as(@Vector(2, u32), [runtime value])
// @as(@Vector(2, u32), [runtime value])
// @as(@Vector(2, u32), [runtime value])
// @as(@Vector(2, u32), [runtime value])
// @as(@Vector(2, u32), [runtime value])
// @as(@Vector(2, u32), [runtime value])
// @as(@Vector(2, u32), [runtime value])
// @as(@Vector(2, u32), .{ undefined, undefined })
// @as(u32, undefined)
// @as(u32, undefined)
// @as(@Vector(2, u32), [runtime value])
// @as(@Vector(2, u32), [runtime value])
// @as(@Vector(2, u32), [runtime value])
// @as(@Vector(2, u32), [runtime value])
// @as(@Vector(2, u32), [runtime value])
// @as(@Vector(2, u32), [runtime value])
// @as(@Vector(2, u32), [runtime value])
// @as(@Vector(2, u32), [runtime value])
// @as(@Vector(2, u32), [runtime value])
// @as(@Vector(2, u32), [runtime value])
// @as(@Vector(2, u32), [runtime value])
// @as(@Vector(2, u32), [runtime value])
// @as(@Vector(2, u32), [runtime value])
// @as(@Vector(2, u32), [runtime value])
// @as(@Vector(2, u32), .{ undefined, undefined })
// @as(i32, undefined)
// @as(i32, undefined)
// @as(@Vector(2, i32), .{ 6, undefined })
// @as(@Vector(2, i32), .{ undefined, 6 })
// @as(@Vector(2, i32), .{ undefined, undefined })
// @as(@Vector(2, i32), .{ 6, undefined })
// @as(@Vector(2, i32), .{ 6, undefined })
// @as(@Vector(2, i32), .{ undefined, undefined })
// @as(@Vector(2, i32), .{ undefined, undefined })
// @as(@Vector(2, i32), .{ undefined, 6 })
// @as(@Vector(2, i32), .{ undefined, undefined })
// @as(@Vector(2, i32), .{ undefined, 6 })
// @as(@Vector(2, i32), .{ undefined, undefined })
// @as(@Vector(2, i32), .{ undefined, undefined })
// @as(@Vector(2, i32), .{ undefined, undefined })
// @as(@Vector(2, i32), .{ undefined, undefined })
// @as(@Vector(2, i32), .{ undefined, undefined })
// @as(i32, undefined)
// @as(i32, undefined)
// @as(@Vector(2, i32), .{ 6, undefined })
// @as(@Vector(2, i32), .{ undefined, 6 })
// @as(@Vector(2, i32), .{ undefined, undefined })
// @as(@Vector(2, i32), .{ 6, undefined })
// @as(@Vector(2, i32), .{ 6, undefined })
// @as(@Vector(2, i32), .{ undefined, undefined })
// @as(@Vector(2, i32), .{ undefined, undefined })
// @as(@Vector(2, i32), .{ undefined, 6 })
// @as(@Vector(2, i32), .{ undefined, undefined })
// @as(@Vector(2, i32), .{ undefined, 6 })
// @as(@Vector(2, i32), .{ undefined, undefined })
// @as(@Vector(2, i32), .{ undefined, undefined })
// @as(@Vector(2, i32), .{ undefined, undefined })
// @as(@Vector(2, i32), .{ undefined, undefined })
// @as(@Vector(2, i32), .{ undefined, undefined })
// @as(i32, undefined)
// @as(i32, undefined)
// @as(@Vector(2, i32), .{ 0, undefined })
// @as(@Vector(2, i32), .{ undefined, 0 })
// @as(@Vector(2, i32), .{ undefined, undefined })
// @as(@Vector(2, i32), .{ 0, undefined })
// @as(@Vector(2, i32), .{ 0, undefined })
// @as(@Vector(2, i32), .{ undefined, undefined })
// @as(@Vector(2, i32), .{ undefined, undefined })
// @as(@Vector(2, i32), .{ undefined, 0 })
// @as(@Vector(2, i32), .{ undefined, undefined })
// @as(@Vector(2, i32), .{ undefined, 0 })
// @as(@Vector(2, i32), .{ undefined, undefined })
// @as(@Vector(2, i32), .{ undefined, undefined })
// @as(@Vector(2, i32), .{ undefined, undefined })
// @as(@Vector(2, i32), .{ undefined, undefined })
// @as(@Vector(2, i32), .{ undefined, undefined })
// @as(i32, undefined)
// @as(i32, undefined)
// @as(@Vector(2, i32), .{ 0, undefined })
// @as(@Vector(2, i32), .{ undefined, 0 })
// @as(@Vector(2, i32), .{ undefined, undefined })
// @as(@Vector(2, i32), .{ 0, undefined })
// @as(@Vector(2, i32), .{ 0, undefined })
// @as(@Vector(2, i32), .{ undefined, undefined })
// @as(@Vector(2, i32), .{ undefined, undefined })
// @as(@Vector(2, i32), .{ undefined, 0 })
// @as(@Vector(2, i32), .{ undefined, undefined })
// @as(@Vector(2, i32), .{ undefined, 0 })
// @as(@Vector(2, i32), .{ undefined, undefined })
// @as(@Vector(2, i32), .{ undefined, undefined })
// @as(@Vector(2, i32), .{ undefined, undefined })
// @as(@Vector(2, i32), .{ undefined, undefined })
// @as(@Vector(2, i32), .{ undefined, undefined })
// @as(i32, undefined)
// @as(i32, undefined)
// @as(@Vector(2, i32), .{ 9, undefined })
// @as(@Vector(2, i32), .{ undefined, 9 })
// @as(@Vector(2, i32), .{ undefined, undefined })
// @as(@Vector(2, i32), .{ 9, undefined })
// @as(@Vector(2, i32), .{ 9, undefined })
// @as(@Vector(2, i32), .{ undefined, undefined })
// @as(@Vector(2, i32), .{ undefined, undefined })
// @as(@Vector(2, i32), .{ undefined, 9 })
// @as(@Vector(2, i32), .{ undefined, undefined })
// @as(@Vector(2, i32), .{ undefined, 9 })
// @as(@Vector(2, i32), .{ undefined, undefined })
// @as(@Vector(2, i32), .{ undefined, undefined })
// @as(@Vector(2, i32), .{ undefined, undefined })
// @as(@Vector(2, i32), .{ undefined, undefined })
// @as(@Vector(2, i32), .{ undefined, undefined })
// @as(i32, undefined)
// @as(i32, undefined)
// @as(@Vector(2, i32), .{ 9, undefined })
// @as(@Vector(2, i32), .{ undefined, 9 })
// @as(@Vector(2, i32), .{ undefined, undefined })
// @as(@Vector(2, i32), .{ 9, undefined })
// @as(@Vector(2, i32), .{ 9, undefined })
// @as(@Vector(2, i32), .{ undefined, undefined })
// @as(@Vector(2, i32), .{ undefined, undefined })
// @as(@Vector(2, i32), .{ undefined, 9 })
// @as(@Vector(2, i32), .{ undefined, undefined })
// @as(@Vector(2, i32), .{ undefined, 9 })
// @as(@Vector(2, i32), .{ undefined, undefined })
// @as(@Vector(2, i32), .{ undefined, undefined })
// @as(@Vector(2, i32), .{ undefined, undefined })
// @as(@Vector(2, i32), .{ undefined, undefined })
// @as(@Vector(2, i32), .{ undefined, undefined })
// @as(i32, undefined)
// @as(i32, undefined)
// @as(@Vector(2, i32), [runtime value])
// @as(@Vector(2, i32), [runtime value])
// @as(@Vector(2, i32), [runtime value])
// @as(@Vector(2, i32), [runtime value])
// @as(@Vector(2, i32), [runtime value])
// @as(@Vector(2, i32), [runtime value])
// @as(@Vector(2, i32), [runtime value])
// @as(@Vector(2, i32), [runtime value])
// @as(@Vector(2, i32), [runtime value])
// @as(@Vector(2, i32), [runtime value])
// @as(@Vector(2, i32), [runtime value])
// @as(@Vector(2, i32), [runtime value])
// @as(@Vector(2, i32), [runtime value])
// @as(@Vector(2, i32), [runtime value])
// @as(@Vector(2, i32), .{ undefined, undefined })
// @as(i32, undefined)
// @as(i32, undefined)
// @as(@Vector(2, i32), [runtime value])
// @as(@Vector(2, i32), [runtime value])
// @as(@Vector(2, i32), [runtime value])
// @as(@Vector(2, i32), [runtime value])
// @as(@Vector(2, i32), [runtime value])
// @as(@Vector(2, i32), [runtime value])
// @as(@Vector(2, i32), [runtime value])
// @as(@Vector(2, i32), [runtime value])
// @as(@Vector(2, i32), [runtime value])
// @as(@Vector(2, i32), [runtime value])
// @as(@Vector(2, i32), [runtime value])
// @as(@Vector(2, i32), [runtime value])
// @as(@Vector(2, i32), [runtime value])
// @as(@Vector(2, i32), [runtime value])
// @as(@Vector(2, i32), .{ undefined, undefined })
// @as(i32, undefined)
// @as(i32, undefined)
// @as(@Vector(2, i32), [runtime value])
// @as(@Vector(2, i32), [runtime value])
// @as(@Vector(2, i32), [runtime value])
// @as(@Vector(2, i32), [runtime value])
// @as(@Vector(2, i32), [runtime value])
// @as(@Vector(2, i32), [runtime value])
// @as(@Vector(2, i32), [runtime value])
// @as(@Vector(2, i32), [runtime value])
// @as(@Vector(2, i32), [runtime value])
// @as(@Vector(2, i32), [runtime value])
// @as(@Vector(2, i32), [runtime value])
// @as(@Vector(2, i32), [runtime value])
// @as(@Vector(2, i32), [runtime value])
// @as(@Vector(2, i32), [runtime value])
// @as(@Vector(2, i32), .{ undefined, undefined })
// @as(i32, undefined)
// @as(i32, undefined)
// @as(@Vector(2, i32), [runtime value])
// @as(@Vector(2, i32), [runtime value])
// @as(@Vector(2, i32), [runtime value])
// @as(@Vector(2, i32), [runtime value])
// @as(@Vector(2, i32), [runtime value])
// @as(@Vector(2, i32), [runtime value])
// @as(@Vector(2, i32), [runtime value])
// @as(@Vector(2, i32), [runtime value])
// @as(@Vector(2, i32), [runtime value])
// @as(@Vector(2, i32), [runtime value])
// @as(@Vector(2, i32), [runtime value])
// @as(@Vector(2, i32), [runtime value])
// @as(@Vector(2, i32), [runtime value])
// @as(@Vector(2, i32), [runtime value])
// @as(@Vector(2, i32), .{ undefined, undefined })
// @as(i32, undefined)
// @as(i32, undefined)
// @as(@Vector(2, i32), [runtime value])
// @as(@Vector(2, i32), [runtime value])
// @as(@Vector(2, i32), [runtime value])
// @as(@Vector(2, i32), [runtime value])
// @as(@Vector(2, i32), [runtime value])
// @as(@Vector(2, i32), [runtime value])
// @as(@Vector(2, i32), [runtime value])
// @as(@Vector(2, i32), [runtime value])
// @as(@Vector(2, i32), [runtime value])
// @as(@Vector(2, i32), [runtime value])
// @as(@Vector(2, i32), [runtime value])
// @as(@Vector(2, i32), [runtime value])
// @as(@Vector(2, i32), [runtime value])
// @as(@Vector(2, i32), [runtime value])
// @as(@Vector(2, i32), .{ undefined, undefined })
// @as(i32, undefined)
// @as(i32, undefined)
// @as(@Vector(2, i32), [runtime value])
// @as(@Vector(2, i32), [runtime value])
// @as(@Vector(2, i32), [runtime value])
// @as(@Vector(2, i32), [runtime value])
// @as(@Vector(2, i32), [runtime value])
// @as(@Vector(2, i32), [runtime value])
// @as(@Vector(2, i32), [runtime value])
// @as(@Vector(2, i32), [runtime value])
// @as(@Vector(2, i32), [runtime value])
// @as(@Vector(2, i32), [runtime value])
// @as(@Vector(2, i32), [runtime value])
// @as(@Vector(2, i32), [runtime value])
// @as(@Vector(2, i32), [runtime value])
// @as(@Vector(2, i32), [runtime value])
// @as(@Vector(2, i32), .{ undefined, undefined })
// @as(u500, undefined)
// @as(u500, undefined)
// @as(@Vector(2, u500), .{ 6, undefined })
// @as(@Vector(2, u500), .{ undefined, 6 })
// @as(@Vector(2, u500), .{ undefined, undefined })
// @as(@Vector(2, u500), .{ 6, undefined })
// @as(@Vector(2, u500), .{ 6, undefined })
// @as(@Vector(2, u500), .{ undefined, undefined })
// @as(@Vector(2, u500), .{ undefined, undefined })
// @as(@Vector(2, u500), .{ undefined, 6 })
// @as(@Vector(2, u500), .{ undefined, undefined })
// @as(@Vector(2, u500), .{ undefined, 6 })
// @as(@Vector(2, u500), .{ undefined, undefined })
// @as(@Vector(2, u500), .{ undefined, undefined })
// @as(@Vector(2, u500), .{ undefined, undefined })
// @as(@Vector(2, u500), .{ undefined, undefined })
// @as(@Vector(2, u500), .{ undefined, undefined })
// @as(u500, undefined)
// @as(u500, undefined)
// @as(@Vector(2, u500), .{ 6, undefined })
// @as(@Vector(2, u500), .{ undefined, 6 })
// @as(@Vector(2, u500), .{ undefined, undefined })
// @as(@Vector(2, u500), .{ 6, undefined })
// @as(@Vector(2, u500), .{ 6, undefined })
// @as(@Vector(2, u500), .{ undefined, undefined })
// @as(@Vector(2, u500), .{ undefined, undefined })
// @as(@Vector(2, u500), .{ undefined, 6 })
// @as(@Vector(2, u500), .{ undefined, undefined })
// @as(@Vector(2, u500), .{ undefined, 6 })
// @as(@Vector(2, u500), .{ undefined, undefined })
// @as(@Vector(2, u500), .{ undefined, undefined })
// @as(@Vector(2, u500), .{ undefined, undefined })
// @as(@Vector(2, u500), .{ undefined, undefined })
// @as(@Vector(2, u500), .{ undefined, undefined })
// @as(u500, undefined)
// @as(u500, undefined)
// @as(@Vector(2, u500), .{ 0, undefined })
// @as(@Vector(2, u500), .{ undefined, 0 })
// @as(@Vector(2, u500), .{ undefined, undefined })
// @as(@Vector(2, u500), .{ 0, undefined })
// @as(@Vector(2, u500), .{ 0, undefined })
// @as(@Vector(2, u500), .{ undefined, undefined })
// @as(@Vector(2, u500), .{ undefined, undefined })
// @as(@Vector(2, u500), .{ undefined, 0 })
// @as(@Vector(2, u500), .{ undefined, undefined })
// @as(@Vector(2, u500), .{ undefined, 0 })
// @as(@Vector(2, u500), .{ undefined, undefined })
// @as(@Vector(2, u500), .{ undefined, undefined })
// @as(@Vector(2, u500), .{ undefined, undefined })
// @as(@Vector(2, u500), .{ undefined, undefined })
// @as(@Vector(2, u500), .{ undefined, undefined })
// @as(u500, undefined)
// @as(u500, undefined)
// @as(@Vector(2, u500), .{ 0, undefined })
// @as(@Vector(2, u500), .{ undefined, 0 })
// @as(@Vector(2, u500), .{ undefined, undefined })
// @as(@Vector(2, u500), .{ 0, undefined })
// @as(@Vector(2, u500), .{ 0, undefined })
// @as(@Vector(2, u500), .{ undefined, undefined })
// @as(@Vector(2, u500), .{ undefined, undefined })
// @as(@Vector(2, u500), .{ undefined, 0 })
// @as(@Vector(2, u500), .{ undefined, undefined })
// @as(@Vector(2, u500), .{ undefined, 0 })
// @as(@Vector(2, u500), .{ undefined, undefined })
// @as(@Vector(2, u500), .{ undefined, undefined })
// @as(@Vector(2, u500), .{ undefined, undefined })
// @as(@Vector(2, u500), .{ undefined, undefined })
// @as(@Vector(2, u500), .{ undefined, undefined })
// @as(u500, undefined)
// @as(u500, undefined)
// @as(@Vector(2, u500), .{ 9, undefined })
// @as(@Vector(2, u500), .{ undefined, 9 })
// @as(@Vector(2, u500), .{ undefined, undefined })
// @as(@Vector(2, u500), .{ 9, undefined })
// @as(@Vector(2, u500), .{ 9, undefined })
// @as(@Vector(2, u500), .{ undefined, undefined })
// @as(@Vector(2, u500), .{ undefined, undefined })
// @as(@Vector(2, u500), .{ undefined, 9 })
// @as(@Vector(2, u500), .{ undefined, undefined })
// @as(@Vector(2, u500), .{ undefined, 9 })
// @as(@Vector(2, u500), .{ undefined, undefined })
// @as(@Vector(2, u500), .{ undefined, undefined })
// @as(@Vector(2, u500), .{ undefined, undefined })
// @as(@Vector(2, u500), .{ undefined, undefined })
// @as(@Vector(2, u500), .{ undefined, undefined })
// @as(u500, undefined)
// @as(u500, undefined)
// @as(@Vector(2, u500), .{ 9, undefined })
// @as(@Vector(2, u500), .{ undefined, 9 })
// @as(@Vector(2, u500), .{ undefined, undefined })
// @as(@Vector(2, u500), .{ 9, undefined })
// @as(@Vector(2, u500), .{ 9, undefined })
// @as(@Vector(2, u500), .{ undefined, undefined })
// @as(@Vector(2, u500), .{ undefined, undefined })
// @as(@Vector(2, u500), .{ undefined, 9 })
// @as(@Vector(2, u500), .{ undefined, undefined })
// @as(@Vector(2, u500), .{ undefined, 9 })
// @as(@Vector(2, u500), .{ undefined, undefined })
// @as(@Vector(2, u500), .{ undefined, undefined })
// @as(@Vector(2, u500), .{ undefined, undefined })
// @as(@Vector(2, u500), .{ undefined, undefined })
// @as(@Vector(2, u500), .{ undefined, undefined })
// @as(u500, undefined)
// @as(u500, undefined)
// @as(@Vector(2, u500), [runtime value])
// @as(@Vector(2, u500), [runtime value])
// @as(@Vector(2, u500), [runtime value])
// @as(@Vector(2, u500), [runtime value])
// @as(@Vector(2, u500), [runtime value])
// @as(@Vector(2, u500), [runtime value])
// @as(@Vector(2, u500), [runtime value])
// @as(@Vector(2, u500), [runtime value])
// @as(@Vector(2, u500), [runtime value])
// @as(@Vector(2, u500), [runtime value])
// @as(@Vector(2, u500), [runtime value])
// @as(@Vector(2, u500), [runtime value])
// @as(@Vector(2, u500), [runtime value])
// @as(@Vector(2, u500), [runtime value])
// @as(@Vector(2, u500), .{ undefined, undefined })
// @as(u500, undefined)
// @as(u500, undefined)
// @as(@Vector(2, u500), [runtime value])
// @as(@Vector(2, u500), [runtime value])
// @as(@Vector(2, u500), [runtime value])
// @as(@Vector(2, u500), [runtime value])
// @as(@Vector(2, u500), [runtime value])
// @as(@Vector(2, u500), [runtime value])
// @as(@Vector(2, u500), [runtime value])
// @as(@Vector(2, u500), [runtime value])
// @as(@Vector(2, u500), [runtime value])
// @as(@Vector(2, u500), [runtime value])
// @as(@Vector(2, u500), [runtime value])
// @as(@Vector(2, u500), [runtime value])
// @as(@Vector(2, u500), [runtime value])
// @as(@Vector(2, u500), [runtime value])
// @as(@Vector(2, u500), .{ undefined, undefined })
// @as(u500, undefined)
// @as(u500, undefined)
// @as(@Vector(2, u500), [runtime value])
// @as(@Vector(2, u500), [runtime value])
// @as(@Vector(2, u500), [runtime value])
// @as(@Vector(2, u500), [runtime value])
// @as(@Vector(2, u500), [runtime value])
// @as(@Vector(2, u500), [runtime value])
// @as(@Vector(2, u500), [runtime value])
// @as(@Vector(2, u500), [runtime value])
// @as(@Vector(2, u500), [runtime value])
// @as(@Vector(2, u500), [runtime value])
// @as(@Vector(2, u500), [runtime value])
// @as(@Vector(2, u500), [runtime value])
// @as(@Vector(2, u500), [runtime value])
// @as(@Vector(2, u500), [runtime value])
// @as(@Vector(2, u500), .{ undefined, undefined })
// @as(u500, undefined)
// @as(u500, undefined)
// @as(@Vector(2, u500), [runtime value])
// @as(@Vector(2, u500), [runtime value])
// @as(@Vector(2, u500), [runtime value])
// @as(@Vector(2, u500), [runtime value])
// @as(@Vector(2, u500), [runtime value])
// @as(@Vector(2, u500), [runtime value])
// @as(@Vector(2, u500), [runtime value])
// @as(@Vector(2, u500), [runtime value])
// @as(@Vector(2, u500), [runtime value])
// @as(@Vector(2, u500), [runtime value])
// @as(@Vector(2, u500), [runtime value])
// @as(@Vector(2, u500), [runtime value])
// @as(@Vector(2, u500), [runtime value])
// @as(@Vector(2, u500), [runtime value])
// @as(@Vector(2, u500), .{ undefined, undefined })
// @as(u500, undefined)
// @as(u500, undefined)
// @as(@Vector(2, u500), [runtime value])
// @as(@Vector(2, u500), [runtime value])
// @as(@Vector(2, u500), [runtime value])
// @as(@Vector(2, u500), [runtime value])
// @as(@Vector(2, u500), [runtime value])
// @as(@Vector(2, u500), [runtime value])
// @as(@Vector(2, u500), [runtime value])
// @as(@Vector(2, u500), [runtime value])
// @as(@Vector(2, u500), [runtime value])
// @as(@Vector(2, u500), [runtime value])
// @as(@Vector(2, u500), [runtime value])
// @as(@Vector(2, u500), [runtime value])
// @as(@Vector(2, u500), [runtime value])
// @as(@Vector(2, u500), [runtime value])
// @as(@Vector(2, u500), .{ undefined, undefined })
// @as(u500, undefined)
// @as(u500, undefined)
// @as(@Vector(2, u500), [runtime value])
// @as(@Vector(2, u500), [runtime value])
// @as(@Vector(2, u500), [runtime value])
// @as(@Vector(2, u500), [runtime value])
// @as(@Vector(2, u500), [runtime value])
// @as(@Vector(2, u500), [runtime value])
// @as(@Vector(2, u500), [runtime value])
// @as(@Vector(2, u500), [runtime value])
// @as(@Vector(2, u500), [runtime value])
// @as(@Vector(2, u500), [runtime value])
// @as(@Vector(2, u500), [runtime value])
// @as(@Vector(2, u500), [runtime value])
// @as(@Vector(2, u500), [runtime value])
// @as(@Vector(2, u500), [runtime value])
// @as(@Vector(2, u500), .{ undefined, undefined })
// @as(i500, undefined)
// @as(i500, undefined)
// @as(@Vector(2, i500), .{ 6, undefined })
// @as(@Vector(2, i500), .{ undefined, 6 })
// @as(@Vector(2, i500), .{ undefined, undefined })
// @as(@Vector(2, i500), .{ 6, undefined })
// @as(@Vector(2, i500), .{ 6, undefined })
// @as(@Vector(2, i500), .{ undefined, undefined })
// @as(@Vector(2, i500), .{ undefined, undefined })
// @as(@Vector(2, i500), .{ undefined, 6 })
// @as(@Vector(2, i500), .{ undefined, undefined })
// @as(@Vector(2, i500), .{ undefined, 6 })
// @as(@Vector(2, i500), .{ undefined, undefined })
// @as(@Vector(2, i500), .{ undefined, undefined })
// @as(@Vector(2, i500), .{ undefined, undefined })
// @as(@Vector(2, i500), .{ undefined, undefined })
// @as(@Vector(2, i500), .{ undefined, undefined })
// @as(i500, undefined)
// @as(i500, undefined)
// @as(@Vector(2, i500), .{ 6, undefined })
// @as(@Vector(2, i500), .{ undefined, 6 })
// @as(@Vector(2, i500), .{ undefined, undefined })
// @as(@Vector(2, i500), .{ 6, undefined })
// @as(@Vector(2, i500), .{ 6, undefined })
// @as(@Vector(2, i500), .{ undefined, undefined })
// @as(@Vector(2, i500), .{ undefined, undefined })
// @as(@Vector(2, i500), .{ undefined, 6 })
// @as(@Vector(2, i500), .{ undefined, undefined })
// @as(@Vector(2, i500), .{ undefined, 6 })
// @as(@Vector(2, i500), .{ undefined, undefined })
// @as(@Vector(2, i500), .{ undefined, undefined })
// @as(@Vector(2, i500), .{ undefined, undefined })
// @as(@Vector(2, i500), .{ undefined, undefined })
// @as(@Vector(2, i500), .{ undefined, undefined })
// @as(i500, undefined)
// @as(i500, undefined)
// @as(@Vector(2, i500), .{ 0, undefined })
// @as(@Vector(2, i500), .{ undefined, 0 })
// @as(@Vector(2, i500), .{ undefined, undefined })
// @as(@Vector(2, i500), .{ 0, undefined })
// @as(@Vector(2, i500), .{ 0, undefined })
// @as(@Vector(2, i500), .{ undefined, undefined })
// @as(@Vector(2, i500), .{ undefined, undefined })
// @as(@Vector(2, i500), .{ undefined, 0 })
// @as(@Vector(2, i500), .{ undefined, undefined })
// @as(@Vector(2, i500), .{ undefined, 0 })
// @as(@Vector(2, i500), .{ undefined, undefined })
// @as(@Vector(2, i500), .{ undefined, undefined })
// @as(@Vector(2, i500), .{ undefined, undefined })
// @as(@Vector(2, i500), .{ undefined, undefined })
// @as(@Vector(2, i500), .{ undefined, undefined })
// @as(i500, undefined)
// @as(i500, undefined)
// @as(@Vector(2, i500), .{ 0, undefined })
// @as(@Vector(2, i500), .{ undefined, 0 })
// @as(@Vector(2, i500), .{ undefined, undefined })
// @as(@Vector(2, i500), .{ 0, undefined })
// @as(@Vector(2, i500), .{ 0, undefined })
// @as(@Vector(2, i500), .{ undefined, undefined })
// @as(@Vector(2, i500), .{ undefined, undefined })
// @as(@Vector(2, i500), .{ undefined, 0 })
// @as(@Vector(2, i500), .{ undefined, undefined })
// @as(@Vector(2, i500), .{ undefined, 0 })
// @as(@Vector(2, i500), .{ undefined, undefined })
// @as(@Vector(2, i500), .{ undefined, undefined })
// @as(@Vector(2, i500), .{ undefined, undefined })
// @as(@Vector(2, i500), .{ undefined, undefined })
// @as(@Vector(2, i500), .{ undefined, undefined })
// @as(i500, undefined)
// @as(i500, undefined)
// @as(@Vector(2, i500), .{ 9, undefined })
// @as(@Vector(2, i500), .{ undefined, 9 })
// @as(@Vector(2, i500), .{ undefined, undefined })
// @as(@Vector(2, i500), .{ 9, undefined })
// @as(@Vector(2, i500), .{ 9, undefined })
// @as(@Vector(2, i500), .{ undefined, undefined })
// @as(@Vector(2, i500), .{ undefined, undefined })
// @as(@Vector(2, i500), .{ undefined, 9 })
// @as(@Vector(2, i500), .{ undefined, undefined })
// @as(@Vector(2, i500), .{ undefined, 9 })
// @as(@Vector(2, i500), .{ undefined, undefined })
// @as(@Vector(2, i500), .{ undefined, undefined })
// @as(@Vector(2, i500), .{ undefined, undefined })
// @as(@Vector(2, i500), .{ undefined, undefined })
// @as(@Vector(2, i500), .{ undefined, undefined })
// @as(i500, undefined)
// @as(i500, undefined)
// @as(@Vector(2, i500), .{ 9, undefined })
// @as(@Vector(2, i500), .{ undefined, 9 })
// @as(@Vector(2, i500), .{ undefined, undefined })
// @as(@Vector(2, i500), .{ 9, undefined })
// @as(@Vector(2, i500), .{ 9, undefined })
// @as(@Vector(2, i500), .{ undefined, undefined })
// @as(@Vector(2, i500), .{ undefined, undefined })
// @as(@Vector(2, i500), .{ undefined, 9 })
// @as(@Vector(2, i500), .{ undefined, undefined })
// @as(@Vector(2, i500), .{ undefined, 9 })
// @as(@Vector(2, i500), .{ undefined, undefined })
// @as(@Vector(2, i500), .{ undefined, undefined })
// @as(@Vector(2, i500), .{ undefined, undefined })
// @as(@Vector(2, i500), .{ undefined, undefined })
// @as(@Vector(2, i500), .{ undefined, undefined })
// @as(i500, undefined)
// @as(i500, undefined)
// @as(@Vector(2, i500), [runtime value])
// @as(@Vector(2, i500), [runtime value])
// @as(@Vector(2, i500), [runtime value])
// @as(@Vector(2, i500), [runtime value])
// @as(@Vector(2, i500), [runtime value])
// @as(@Vector(2, i500), [runtime value])
// @as(@Vector(2, i500), [runtime value])
// @as(@Vector(2, i500), [runtime value])
// @as(@Vector(2, i500), [runtime value])
// @as(@Vector(2, i500), [runtime value])
// @as(@Vector(2, i500), [runtime value])
// @as(@Vector(2, i500), [runtime value])
// @as(@Vector(2, i500), [runtime value])
// @as(@Vector(2, i500), [runtime value])
// @as(@Vector(2, i500), .{ undefined, undefined })
// @as(i500, undefined)
// @as(i500, undefined)
// @as(@Vector(2, i500), [runtime value])
// @as(@Vector(2, i500), [runtime value])
// @as(@Vector(2, i500), [runtime value])
// @as(@Vector(2, i500), [runtime value])
// @as(@Vector(2, i500), [runtime value])
// @as(@Vector(2, i500), [runtime value])
// @as(@Vector(2, i500), [runtime value])
// @as(@Vector(2, i500), [runtime value])
// @as(@Vector(2, i500), [runtime value])
// @as(@Vector(2, i500), [runtime value])
// @as(@Vector(2, i500), [runtime value])
// @as(@Vector(2, i500), [runtime value])
// @as(@Vector(2, i500), [runtime value])
// @as(@Vector(2, i500), [runtime value])
// @as(@Vector(2, i500), .{ undefined, undefined })
// @as(i500, undefined)
// @as(i500, undefined)
// @as(@Vector(2, i500), [runtime value])
// @as(@Vector(2, i500), [runtime value])
// @as(@Vector(2, i500), [runtime value])
// @as(@Vector(2, i500), [runtime value])
// @as(@Vector(2, i500), [runtime value])
// @as(@Vector(2, i500), [runtime value])
// @as(@Vector(2, i500), [runtime value])
// @as(@Vector(2, i500), [runtime value])
// @as(@Vector(2, i500), [runtime value])
// @as(@Vector(2, i500), [runtime value])
// @as(@Vector(2, i500), [runtime value])
// @as(@Vector(2, i500), [runtime value])
// @as(@Vector(2, i500), [runtime value])
// @as(@Vector(2, i500), [runtime value])
// @as(@Vector(2, i500), .{ undefined, undefined })
// @as(i500, undefined)
// @as(i500, undefined)
// @as(@Vector(2, i500), [runtime value])
// @as(@Vector(2, i500), [runtime value])
// @as(@Vector(2, i500), [runtime value])
// @as(@Vector(2, i500), [runtime value])
// @as(@Vector(2, i500), [runtime value])
// @as(@Vector(2, i500), [runtime value])
// @as(@Vector(2, i500), [runtime value])
// @as(@Vector(2, i500), [runtime value])
// @as(@Vector(2, i500), [runtime value])
// @as(@Vector(2, i500), [runtime value])
// @as(@Vector(2, i500), [runtime value])
// @as(@Vector(2, i500), [runtime value])
// @as(@Vector(2, i500), [runtime value])
// @as(@Vector(2, i500), [runtime value])
// @as(@Vector(2, i500), .{ undefined, undefined })
// @as(i500, undefined)
// @as(i500, undefined)
// @as(@Vector(2, i500), [runtime value])
// @as(@Vector(2, i500), [runtime value])
// @as(@Vector(2, i500), [runtime value])
// @as(@Vector(2, i500), [runtime value])
// @as(@Vector(2, i500), [runtime value])
// @as(@Vector(2, i500), [runtime value])
// @as(@Vector(2, i500), [runtime value])
// @as(@Vector(2, i500), [runtime value])
// @as(@Vector(2, i500), [runtime value])
// @as(@Vector(2, i500), [runtime value])
// @as(@Vector(2, i500), [runtime value])
// @as(@Vector(2, i500), [runtime value])
// @as(@Vector(2, i500), [runtime value])
// @as(@Vector(2, i500), [runtime value])
// @as(@Vector(2, i500), .{ undefined, undefined })
// @as(i500, undefined)
// @as(i500, undefined)
// @as(@Vector(2, i500), [runtime value])
// @as(@Vector(2, i500), [runtime value])
// @as(@Vector(2, i500), [runtime value])
// @as(@Vector(2, i500), [runtime value])
// @as(@Vector(2, i500), [runtime value])
// @as(@Vector(2, i500), [runtime value])
// @as(@Vector(2, i500), [runtime value])
// @as(@Vector(2, i500), [runtime value])
// @as(@Vector(2, i500), [runtime value])
// @as(@Vector(2, i500), [runtime value])
// @as(@Vector(2, i500), [runtime value])
// @as(@Vector(2, i500), [runtime value])
// @as(@Vector(2, i500), [runtime value])
// @as(@Vector(2, i500), [runtime value])
// @as(@Vector(2, i500), .{ undefined, undefined })
// @as(f16, undefined)
// @as(f16, undefined)
// @as(@Vector(2, f16), .{ 6, undefined })
// @as(@Vector(2, f16), .{ undefined, 6 })
// @as(@Vector(2, f16), .{ undefined, undefined })
// @as(@Vector(2, f16), .{ 6, undefined })
// @as(@Vector(2, f16), .{ 6, undefined })
// @as(@Vector(2, f16), .{ undefined, undefined })
// @as(@Vector(2, f16), .{ undefined, undefined })
// @as(@Vector(2, f16), .{ undefined, 6 })
// @as(@Vector(2, f16), .{ undefined, undefined })
// @as(@Vector(2, f16), .{ undefined, 6 })
// @as(@Vector(2, f16), .{ undefined, undefined })
// @as(@Vector(2, f16), .{ undefined, undefined })
// @as(@Vector(2, f16), .{ undefined, undefined })
// @as(@Vector(2, f16), .{ undefined, undefined })
// @as(@Vector(2, f16), .{ undefined, undefined })
// @as(f16, undefined)
// @as(f16, undefined)
// @as(@Vector(2, f16), .{ 0, undefined })
// @as(@Vector(2, f16), .{ undefined, 0 })
// @as(@Vector(2, f16), .{ undefined, undefined })
// @as(@Vector(2, f16), .{ 0, undefined })
// @as(@Vector(2, f16), .{ 0, undefined })
// @as(@Vector(2, f16), .{ undefined, undefined })
// @as(@Vector(2, f16), .{ undefined, undefined })
// @as(@Vector(2, f16), .{ undefined, 0 })
// @as(@Vector(2, f16), .{ undefined, undefined })
// @as(@Vector(2, f16), .{ undefined, 0 })
// @as(@Vector(2, f16), .{ undefined, undefined })
// @as(@Vector(2, f16), .{ undefined, undefined })
// @as(@Vector(2, f16), .{ undefined, undefined })
// @as(@Vector(2, f16), .{ undefined, undefined })
// @as(@Vector(2, f16), .{ undefined, undefined })
// @as(f16, undefined)
// @as(f16, undefined)
// @as(@Vector(2, f16), .{ 9, undefined })
// @as(@Vector(2, f16), .{ undefined, 9 })
// @as(@Vector(2, f16), .{ undefined, undefined })
// @as(@Vector(2, f16), .{ 9, undefined })
// @as(@Vector(2, f16), .{ 9, undefined })
// @as(@Vector(2, f16), .{ undefined, undefined })
// @as(@Vector(2, f16), .{ undefined, undefined })
// @as(@Vector(2, f16), .{ undefined, 9 })
// @as(@Vector(2, f16), .{ undefined, undefined })
// @as(@Vector(2, f16), .{ undefined, 9 })
// @as(@Vector(2, f16), .{ undefined, undefined })
// @as(@Vector(2, f16), .{ undefined, undefined })
// @as(@Vector(2, f16), .{ undefined, undefined })
// @as(@Vector(2, f16), .{ undefined, undefined })
// @as(@Vector(2, f16), .{ undefined, undefined })
// @as(f16, undefined)
// @as(@Vector(2, f16), .{ -3, undefined })
// @as(@Vector(2, f16), .{ undefined, -3 })
// @as(@Vector(2, f16), .{ undefined, undefined })
// @as(f16, undefined)
// @as(f16, undefined)
// @as(@Vector(2, f16), [runtime value])
// @as(@Vector(2, f16), [runtime value])
// @as(@Vector(2, f16), [runtime value])
// @as(@Vector(2, f16), [runtime value])
// @as(@Vector(2, f16), [runtime value])
// @as(@Vector(2, f16), [runtime value])
// @as(@Vector(2, f16), [runtime value])
// @as(@Vector(2, f16), [runtime value])
// @as(@Vector(2, f16), [runtime value])
// @as(@Vector(2, f16), [runtime value])
// @as(@Vector(2, f16), [runtime value])
// @as(@Vector(2, f16), [runtime value])
// @as(@Vector(2, f16), [runtime value])
// @as(@Vector(2, f16), [runtime value])
// @as(@Vector(2, f16), .{ undefined, undefined })
// @as(f16, undefined)
// @as(f16, undefined)
// @as(@Vector(2, f16), [runtime value])
// @as(@Vector(2, f16), [runtime value])
// @as(@Vector(2, f16), [runtime value])
// @as(@Vector(2, f16), [runtime value])
// @as(@Vector(2, f16), [runtime value])
// @as(@Vector(2, f16), [runtime value])
// @as(@Vector(2, f16), [runtime value])
// @as(@Vector(2, f16), [runtime value])
// @as(@Vector(2, f16), [runtime value])
// @as(@Vector(2, f16), [runtime value])
// @as(@Vector(2, f16), [runtime value])
// @as(@Vector(2, f16), [runtime value])
// @as(@Vector(2, f16), [runtime value])
// @as(@Vector(2, f16), [runtime value])
// @as(@Vector(2, f16), .{ undefined, undefined })
// @as(f16, undefined)
// @as(f16, undefined)
// @as(@Vector(2, f16), [runtime value])
// @as(@Vector(2, f16), [runtime value])
// @as(@Vector(2, f16), [runtime value])
// @as(@Vector(2, f16), [runtime value])
// @as(@Vector(2, f16), [runtime value])
// @as(@Vector(2, f16), [runtime value])
// @as(@Vector(2, f16), [runtime value])
// @as(@Vector(2, f16), [runtime value])
// @as(@Vector(2, f16), [runtime value])
// @as(@Vector(2, f16), [runtime value])
// @as(@Vector(2, f16), [runtime value])
// @as(@Vector(2, f16), [runtime value])
// @as(@Vector(2, f16), [runtime value])
// @as(@Vector(2, f16), [runtime value])
// @as(@Vector(2, f16), .{ undefined, undefined })
// @as(f16, undefined)
// @as(@Vector(2, f16), [runtime value])
// @as(@Vector(2, f16), [runtime value])
// @as(@Vector(2, f16), .{ undefined, undefined })
// @as(f32, undefined)
// @as(f32, undefined)
// @as(@Vector(2, f32), .{ 6, undefined })
// @as(@Vector(2, f32), .{ undefined, 6 })
// @as(@Vector(2, f32), .{ undefined, undefined })
// @as(@Vector(2, f32), .{ 6, undefined })
// @as(@Vector(2, f32), .{ 6, undefined })
// @as(@Vector(2, f32), .{ undefined, undefined })
// @as(@Vector(2, f32), .{ undefined, undefined })
// @as(@Vector(2, f32), .{ undefined, 6 })
// @as(@Vector(2, f32), .{ undefined, undefined })
// @as(@Vector(2, f32), .{ undefined, 6 })
// @as(@Vector(2, f32), .{ undefined, undefined })
// @as(@Vector(2, f32), .{ undefined, undefined })
// @as(@Vector(2, f32), .{ undefined, undefined })
// @as(@Vector(2, f32), .{ undefined, undefined })
// @as(@Vector(2, f32), .{ undefined, undefined })
// @as(f32, undefined)
// @as(f32, undefined)
// @as(@Vector(2, f32), .{ 0, undefined })
// @as(@Vector(2, f32), .{ undefined, 0 })
// @as(@Vector(2, f32), .{ undefined, undefined })
// @as(@Vector(2, f32), .{ 0, undefined })
// @as(@Vector(2, f32), .{ 0, undefined })
// @as(@Vector(2, f32), .{ undefined, undefined })
// @as(@Vector(2, f32), .{ undefined, undefined })
// @as(@Vector(2, f32), .{ undefined, 0 })
// @as(@Vector(2, f32), .{ undefined, undefined })
// @as(@Vector(2, f32), .{ undefined, 0 })
// @as(@Vector(2, f32), .{ undefined, undefined })
// @as(@Vector(2, f32), .{ undefined, undefined })
// @as(@Vector(2, f32), .{ undefined, undefined })
// @as(@Vector(2, f32), .{ undefined, undefined })
// @as(@Vector(2, f32), .{ undefined, undefined })
// @as(f32, undefined)
// @as(f32, undefined)
// @as(@Vector(2, f32), .{ 9, undefined })
// @as(@Vector(2, f32), .{ undefined, 9 })
// @as(@Vector(2, f32), .{ undefined, undefined })
// @as(@Vector(2, f32), .{ 9, undefined })
// @as(@Vector(2, f32), .{ 9, undefined })
// @as(@Vector(2, f32), .{ undefined, undefined })
// @as(@Vector(2, f32), .{ undefined, undefined })
// @as(@Vector(2, f32), .{ undefined, 9 })
// @as(@Vector(2, f32), .{ undefined, undefined })
// @as(@Vector(2, f32), .{ undefined, 9 })
// @as(@Vector(2, f32), .{ undefined, undefined })
// @as(@Vector(2, f32), .{ undefined, undefined })
// @as(@Vector(2, f32), .{ undefined, undefined })
// @as(@Vector(2, f32), .{ undefined, undefined })
// @as(@Vector(2, f32), .{ undefined, undefined })
// @as(f32, undefined)
// @as(@Vector(2, f32), .{ -3, undefined })
// @as(@Vector(2, f32), .{ undefined, -3 })
// @as(@Vector(2, f32), .{ undefined, undefined })
// @as(f32, undefined)
// @as(f32, undefined)
// @as(@Vector(2, f32), [runtime value])
// @as(@Vector(2, f32), [runtime value])
// @as(@Vector(2, f32), [runtime value])
// @as(@Vector(2, f32), [runtime value])
// @as(@Vector(2, f32), [runtime value])
// @as(@Vector(2, f32), [runtime value])
// @as(@Vector(2, f32), [runtime value])
// @as(@Vector(2, f32), [runtime value])
// @as(@Vector(2, f32), [runtime value])
// @as(@Vector(2, f32), [runtime value])
// @as(@Vector(2, f32), [runtime value])
// @as(@Vector(2, f32), [runtime value])
// @as(@Vector(2, f32), [runtime value])
// @as(@Vector(2, f32), [runtime value])
// @as(@Vector(2, f32), .{ undefined, undefined })
// @as(f32, undefined)
// @as(f32, undefined)
// @as(@Vector(2, f32), [runtime value])
// @as(@Vector(2, f32), [runtime value])
// @as(@Vector(2, f32), [runtime value])
// @as(@Vector(2, f32), [runtime value])
// @as(@Vector(2, f32), [runtime value])
// @as(@Vector(2, f32), [runtime value])
// @as(@Vector(2, f32), [runtime value])
// @as(@Vector(2, f32), [runtime value])
// @as(@Vector(2, f32), [runtime value])
// @as(@Vector(2, f32), [runtime value])
// @as(@Vector(2, f32), [runtime value])
// @as(@Vector(2, f32), [runtime value])
// @as(@Vector(2, f32), [runtime value])
// @as(@Vector(2, f32), [runtime value])
// @as(@Vector(2, f32), .{ undefined, undefined })
// @as(f32, undefined)
// @as(f32, undefined)
// @as(@Vector(2, f32), [runtime value])
// @as(@Vector(2, f32), [runtime value])
// @as(@Vector(2, f32), [runtime value])
// @as(@Vector(2, f32), [runtime value])
// @as(@Vector(2, f32), [runtime value])
// @as(@Vector(2, f32), [runtime value])
// @as(@Vector(2, f32), [runtime value])
// @as(@Vector(2, f32), [runtime value])
// @as(@Vector(2, f32), [runtime value])
// @as(@Vector(2, f32), [runtime value])
// @as(@Vector(2, f32), [runtime value])
// @as(@Vector(2, f32), [runtime value])
// @as(@Vector(2, f32), [runtime value])
// @as(@Vector(2, f32), [runtime value])
// @as(@Vector(2, f32), .{ undefined, undefined })
// @as(f32, undefined)
// @as(@Vector(2, f32), [runtime value])
// @as(@Vector(2, f32), [runtime value])
// @as(@Vector(2, f32), .{ undefined, undefined })
// @as(f64, undefined)
// @as(f64, undefined)
// @as(@Vector(2, f64), .{ 6, undefined })
// @as(@Vector(2, f64), .{ undefined, 6 })
// @as(@Vector(2, f64), .{ undefined, undefined })
// @as(@Vector(2, f64), .{ 6, undefined })
// @as(@Vector(2, f64), .{ 6, undefined })
// @as(@Vector(2, f64), .{ undefined, undefined })
// @as(@Vector(2, f64), .{ undefined, undefined })
// @as(@Vector(2, f64), .{ undefined, 6 })
// @as(@Vector(2, f64), .{ undefined, undefined })
// @as(@Vector(2, f64), .{ undefined, 6 })
// @as(@Vector(2, f64), .{ undefined, undefined })
// @as(@Vector(2, f64), .{ undefined, undefined })
// @as(@Vector(2, f64), .{ undefined, undefined })
// @as(@Vector(2, f64), .{ undefined, undefined })
// @as(@Vector(2, f64), .{ undefined, undefined })
// @as(f64, undefined)
// @as(f64, undefined)
// @as(@Vector(2, f64), .{ 0, undefined })
// @as(@Vector(2, f64), .{ undefined, 0 })
// @as(@Vector(2, f64), .{ undefined, undefined })
// @as(@Vector(2, f64), .{ 0, undefined })
// @as(@Vector(2, f64), .{ 0, undefined })
// @as(@Vector(2, f64), .{ undefined, undefined })
// @as(@Vector(2, f64), .{ undefined, undefined })
// @as(@Vector(2, f64), .{ undefined, 0 })
// @as(@Vector(2, f64), .{ undefined, undefined })
// @as(@Vector(2, f64), .{ undefined, 0 })
// @as(@Vector(2, f64), .{ undefined, undefined })
// @as(@Vector(2, f64), .{ undefined, undefined })
// @as(@Vector(2, f64), .{ undefined, undefined })
// @as(@Vector(2, f64), .{ undefined, undefined })
// @as(@Vector(2, f64), .{ undefined, undefined })
// @as(f64, undefined)
// @as(f64, undefined)
// @as(@Vector(2, f64), .{ 9, undefined })
// @as(@Vector(2, f64), .{ undefined, 9 })
// @as(@Vector(2, f64), .{ undefined, undefined })
// @as(@Vector(2, f64), .{ 9, undefined })
// @as(@Vector(2, f64), .{ 9, undefined })
// @as(@Vector(2, f64), .{ undefined, undefined })
// @as(@Vector(2, f64), .{ undefined, undefined })
// @as(@Vector(2, f64), .{ undefined, 9 })
// @as(@Vector(2, f64), .{ undefined, undefined })
// @as(@Vector(2, f64), .{ undefined, 9 })
// @as(@Vector(2, f64), .{ undefined, undefined })
// @as(@Vector(2, f64), .{ undefined, undefined })
// @as(@Vector(2, f64), .{ undefined, undefined })
// @as(@Vector(2, f64), .{ undefined, undefined })
// @as(@Vector(2, f64), .{ undefined, undefined })
// @as(f64, undefined)
// @as(@Vector(2, f64), .{ -3, undefined })
// @as(@Vector(2, f64), .{ undefined, -3 })
// @as(@Vector(2, f64), .{ undefined, undefined })
// @as(f64, undefined)
// @as(f64, undefined)
// @as(@Vector(2, f64), [runtime value])
// @as(@Vector(2, f64), [runtime value])
// @as(@Vector(2, f64), [runtime value])
// @as(@Vector(2, f64), [runtime value])
// @as(@Vector(2, f64), [runtime value])
// @as(@Vector(2, f64), [runtime value])
// @as(@Vector(2, f64), [runtime value])
// @as(@Vector(2, f64), [runtime value])
// @as(@Vector(2, f64), [runtime value])
// @as(@Vector(2, f64), [runtime value])
// @as(@Vector(2, f64), [runtime value])
// @as(@Vector(2, f64), [runtime value])
// @as(@Vector(2, f64), [runtime value])
// @as(@Vector(2, f64), [runtime value])
// @as(@Vector(2, f64), .{ undefined, undefined })
// @as(f64, undefined)
// @as(f64, undefined)
// @as(@Vector(2, f64), [runtime value])
// @as(@Vector(2, f64), [runtime value])
// @as(@Vector(2, f64), [runtime value])
// @as(@Vector(2, f64), [runtime value])
// @as(@Vector(2, f64), [runtime value])
// @as(@Vector(2, f64), [runtime value])
// @as(@Vector(2, f64), [runtime value])
// @as(@Vector(2, f64), [runtime value])
// @as(@Vector(2, f64), [runtime value])
// @as(@Vector(2, f64), [runtime value])
// @as(@Vector(2, f64), [runtime value])
// @as(@Vector(2, f64), [runtime value])
// @as(@Vector(2, f64), [runtime value])
// @as(@Vector(2, f64), [runtime value])
// @as(@Vector(2, f64), .{ undefined, undefined })
// @as(f64, undefined)
// @as(f64, undefined)
// @as(@Vector(2, f64), [runtime value])
// @as(@Vector(2, f64), [runtime value])
// @as(@Vector(2, f64), [runtime value])
// @as(@Vector(2, f64), [runtime value])
// @as(@Vector(2, f64), [runtime value])
// @as(@Vector(2, f64), [runtime value])
// @as(@Vector(2, f64), [runtime value])
// @as(@Vector(2, f64), [runtime value])
// @as(@Vector(2, f64), [runtime value])
// @as(@Vector(2, f64), [runtime value])
// @as(@Vector(2, f64), [runtime value])
// @as(@Vector(2, f64), [runtime value])
// @as(@Vector(2, f64), [runtime value])
// @as(@Vector(2, f64), [runtime value])
// @as(@Vector(2, f64), .{ undefined, undefined })
// @as(f64, undefined)
// @as(@Vector(2, f64), [runtime value])
// @as(@Vector(2, f64), [runtime value])
// @as(@Vector(2, f64), .{ undefined, undefined })
// @as(f80, undefined)
// @as(f80, undefined)
// @as(@Vector(2, f80), .{ 6, undefined })
// @as(@Vector(2, f80), .{ undefined, 6 })
// @as(@Vector(2, f80), .{ undefined, undefined })
// @as(@Vector(2, f80), .{ 6, undefined })
// @as(@Vector(2, f80), .{ 6, undefined })
// @as(@Vector(2, f80), .{ undefined, undefined })
// @as(@Vector(2, f80), .{ undefined, undefined })
// @as(@Vector(2, f80), .{ undefined, 6 })
// @as(@Vector(2, f80), .{ undefined, undefined })
// @as(@Vector(2, f80), .{ undefined, 6 })
// @as(@Vector(2, f80), .{ undefined, undefined })
// @as(@Vector(2, f80), .{ undefined, undefined })
// @as(@Vector(2, f80), .{ undefined, undefined })
// @as(@Vector(2, f80), .{ undefined, undefined })
// @as(@Vector(2, f80), .{ undefined, undefined })
// @as(f80, undefined)
// @as(f80, undefined)
// @as(@Vector(2, f80), .{ 0, undefined })
// @as(@Vector(2, f80), .{ undefined, 0 })
// @as(@Vector(2, f80), .{ undefined, undefined })
// @as(@Vector(2, f80), .{ 0, undefined })
// @as(@Vector(2, f80), .{ 0, undefined })
// @as(@Vector(2, f80), .{ undefined, undefined })
// @as(@Vector(2, f80), .{ undefined, undefined })
// @as(@Vector(2, f80), .{ undefined, 0 })
// @as(@Vector(2, f80), .{ undefined, undefined })
// @as(@Vector(2, f80), .{ undefined, 0 })
// @as(@Vector(2, f80), .{ undefined, undefined })
// @as(@Vector(2, f80), .{ undefined, undefined })
// @as(@Vector(2, f80), .{ undefined, undefined })
// @as(@Vector(2, f80), .{ undefined, undefined })
// @as(@Vector(2, f80), .{ undefined, undefined })
// @as(f80, undefined)
// @as(f80, undefined)
// @as(@Vector(2, f80), .{ 9, undefined })
// @as(@Vector(2, f80), .{ undefined, 9 })
// @as(@Vector(2, f80), .{ undefined, undefined })
// @as(@Vector(2, f80), .{ 9, undefined })
// @as(@Vector(2, f80), .{ 9, undefined })
// @as(@Vector(2, f80), .{ undefined, undefined })
// @as(@Vector(2, f80), .{ undefined, undefined })
// @as(@Vector(2, f80), .{ undefined, 9 })
// @as(@Vector(2, f80), .{ undefined, undefined })
// @as(@Vector(2, f80), .{ undefined, 9 })
// @as(@Vector(2, f80), .{ undefined, undefined })
// @as(@Vector(2, f80), .{ undefined, undefined })
// @as(@Vector(2, f80), .{ undefined, undefined })
// @as(@Vector(2, f80), .{ undefined, undefined })
// @as(@Vector(2, f80), .{ undefined, undefined })
// @as(f80, undefined)
// @as(@Vector(2, f80), .{ -3, undefined })
// @as(@Vector(2, f80), .{ undefined, -3 })
// @as(@Vector(2, f80), .{ undefined, undefined })
// @as(f80, undefined)
// @as(f80, undefined)
// @as(@Vector(2, f80), [runtime value])
// @as(@Vector(2, f80), [runtime value])
// @as(@Vector(2, f80), [runtime value])
// @as(@Vector(2, f80), [runtime value])
// @as(@Vector(2, f80), [runtime value])
// @as(@Vector(2, f80), [runtime value])
// @as(@Vector(2, f80), [runtime value])
// @as(@Vector(2, f80), [runtime value])
// @as(@Vector(2, f80), [runtime value])
// @as(@Vector(2, f80), [runtime value])
// @as(@Vector(2, f80), [runtime value])
// @as(@Vector(2, f80), [runtime value])
// @as(@Vector(2, f80), [runtime value])
// @as(@Vector(2, f80), [runtime value])
// @as(@Vector(2, f80), .{ undefined, undefined })
// @as(f80, undefined)
// @as(f80, undefined)
// @as(@Vector(2, f80), [runtime value])
// @as(@Vector(2, f80), [runtime value])
// @as(@Vector(2, f80), [runtime value])
// @as(@Vector(2, f80), [runtime value])
// @as(@Vector(2, f80), [runtime value])
// @as(@Vector(2, f80), [runtime value])
// @as(@Vector(2, f80), [runtime value])
// @as(@Vector(2, f80), [runtime value])
// @as(@Vector(2, f80), [runtime value])
// @as(@Vector(2, f80), [runtime value])
// @as(@Vector(2, f80), [runtime value])
// @as(@Vector(2, f80), [runtime value])
// @as(@Vector(2, f80), [runtime value])
// @as(@Vector(2, f80), [runtime value])
// @as(@Vector(2, f80), .{ undefined, undefined })
// @as(f80, undefined)
// @as(f80, undefined)
// @as(@Vector(2, f80), [runtime value])
// @as(@Vector(2, f80), [runtime value])
// @as(@Vector(2, f80), [runtime value])
// @as(@Vector(2, f80), [runtime value])
// @as(@Vector(2, f80), [runtime value])
// @as(@Vector(2, f80), [runtime value])
// @as(@Vector(2, f80), [runtime value])
// @as(@Vector(2, f80), [runtime value])
// @as(@Vector(2, f80), [runtime value])
// @as(@Vector(2, f80), [runtime value])
// @as(@Vector(2, f80), [runtime value])
// @as(@Vector(2, f80), [runtime value])
// @as(@Vector(2, f80), [runtime value])
// @as(@Vector(2, f80), [runtime value])
// @as(@Vector(2, f80), .{ undefined, undefined })
// @as(f80, undefined)
// @as(@Vector(2, f80), [runtime value])
// @as(@Vector(2, f80), [runtime value])
// @as(@Vector(2, f80), .{ undefined, undefined })
// @as(f128, undefined)
// @as(f128, undefined)
// @as(@Vector(2, f128), .{ 6, undefined })
// @as(@Vector(2, f128), .{ undefined, 6 })
// @as(@Vector(2, f128), .{ undefined, undefined })
// @as(@Vector(2, f128), .{ 6, undefined })
// @as(@Vector(2, f128), .{ 6, undefined })
// @as(@Vector(2, f128), .{ undefined, undefined })
// @as(@Vector(2, f128), .{ undefined, undefined })
// @as(@Vector(2, f128), .{ undefined, 6 })
// @as(@Vector(2, f128), .{ undefined, undefined })
// @as(@Vector(2, f128), .{ undefined, 6 })
// @as(@Vector(2, f128), .{ undefined, undefined })
// @as(@Vector(2, f128), .{ undefined, undefined })
// @as(@Vector(2, f128), .{ undefined, undefined })
// @as(@Vector(2, f128), .{ undefined, undefined })
// @as(@Vector(2, f128), .{ undefined, undefined })
// @as(f128, undefined)
// @as(f128, undefined)
// @as(@Vector(2, f128), .{ 0, undefined })
// @as(@Vector(2, f128), .{ undefined, 0 })
// @as(@Vector(2, f128), .{ undefined, undefined })
// @as(@Vector(2, f128), .{ 0, undefined })
// @as(@Vector(2, f128), .{ 0, undefined })
// @as(@Vector(2, f128), .{ undefined, undefined })
// @as(@Vector(2, f128), .{ undefined, undefined })
// @as(@Vector(2, f128), .{ undefined, 0 })
// @as(@Vector(2, f128), .{ undefined, undefined })
// @as(@Vector(2, f128), .{ undefined, 0 })
// @as(@Vector(2, f128), .{ undefined, undefined })
// @as(@Vector(2, f128), .{ undefined, undefined })
// @as(@Vector(2, f128), .{ undefined, undefined })
// @as(@Vector(2, f128), .{ undefined, undefined })
// @as(@Vector(2, f128), .{ undefined, undefined })
// @as(f128, undefined)
// @as(f128, undefined)
// @as(@Vector(2, f128), .{ 9, undefined })
// @as(@Vector(2, f128), .{ undefined, 9 })
// @as(@Vector(2, f128), .{ undefined, undefined })
// @as(@Vector(2, f128), .{ 9, undefined })
// @as(@Vector(2, f128), .{ 9, undefined })
// @as(@Vector(2, f128), .{ undefined, undefined })
// @as(@Vector(2, f128), .{ undefined, undefined })
// @as(@Vector(2, f128), .{ undefined, 9 })
// @as(@Vector(2, f128), .{ undefined, undefined })
// @as(@Vector(2, f128), .{ undefined, 9 })
// @as(@Vector(2, f128), .{ undefined, undefined })
// @as(@Vector(2, f128), .{ undefined, undefined })
// @as(@Vector(2, f128), .{ undefined, undefined })
// @as(@Vector(2, f128), .{ undefined, undefined })
// @as(@Vector(2, f128), .{ undefined, undefined })
// @as(f128, undefined)
// @as(@Vector(2, f128), .{ -3, undefined })
// @as(@Vector(2, f128), .{ undefined, -3 })
// @as(@Vector(2, f128), .{ undefined, undefined })
// @as(f128, undefined)
// @as(f128, undefined)
// @as(@Vector(2, f128), [runtime value])
// @as(@Vector(2, f128), [runtime value])
// @as(@Vector(2, f128), [runtime value])
// @as(@Vector(2, f128), [runtime value])
// @as(@Vector(2, f128), [runtime value])
// @as(@Vector(2, f128), [runtime value])
// @as(@Vector(2, f128), [runtime value])
// @as(@Vector(2, f128), [runtime value])
// @as(@Vector(2, f128), [runtime value])
// @as(@Vector(2, f128), [runtime value])
// @as(@Vector(2, f128), [runtime value])
// @as(@Vector(2, f128), [runtime value])
// @as(@Vector(2, f128), [runtime value])
// @as(@Vector(2, f128), [runtime value])
// @as(@Vector(2, f128), .{ undefined, undefined })
// @as(f128, undefined)
// @as(f128, undefined)
// @as(@Vector(2, f128), [runtime value])
// @as(@Vector(2, f128), [runtime value])
// @as(@Vector(2, f128), [runtime value])
// @as(@Vector(2, f128), [runtime value])
// @as(@Vector(2, f128), [runtime value])
// @as(@Vector(2, f128), [runtime value])
// @as(@Vector(2, f128), [runtime value])
// @as(@Vector(2, f128), [runtime value])
// @as(@Vector(2, f128), [runtime value])
// @as(@Vector(2, f128), [runtime value])
// @as(@Vector(2, f128), [runtime value])
// @as(@Vector(2, f128), [runtime value])
// @as(@Vector(2, f128), [runtime value])
// @as(@Vector(2, f128), [runtime value])
// @as(@Vector(2, f128), .{ undefined, undefined })
// @as(f128, undefined)
// @as(f128, undefined)
// @as(@Vector(2, f128), [runtime value])
// @as(@Vector(2, f128), [runtime value])
// @as(@Vector(2, f128), [runtime value])
// @as(@Vector(2, f128), [runtime value])
// @as(@Vector(2, f128), [runtime value])
// @as(@Vector(2, f128), [runtime value])
// @as(@Vector(2, f128), [runtime value])
// @as(@Vector(2, f128), [runtime value])
// @as(@Vector(2, f128), [runtime value])
// @as(@Vector(2, f128), [runtime value])
// @as(@Vector(2, f128), [runtime value])
// @as(@Vector(2, f128), [runtime value])
// @as(@Vector(2, f128), [runtime value])
// @as(@Vector(2, f128), [runtime value])
// @as(@Vector(2, f128), .{ undefined, undefined })
// @as(f128, undefined)
// @as(@Vector(2, f128), [runtime value])
// @as(@Vector(2, f128), [runtime value])
// @as(@Vector(2, f128), .{ undefined, undefined })
